@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -46,6 +46,8 @@ public:
     void renamed(char const* name) override;
     void cursor_image_removed() override;
     void placed_relative(geometry::Rectangle const& placement) override;
+    void input_consumed(MirEvent const* event) override;
+    void start_drag_and_drop(std::vector<uint8_t> const& handle) override;
 
 protected:
     NullSurfaceObserver(NullSurfaceObserver const&) = delete;
