@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,6 +27,7 @@
 
 #include <glm/glm.hpp>
 #include <string>
+#include <vector>
 
 namespace mir
 {
@@ -61,6 +62,8 @@ public:
     virtual void renamed(char const* name) = 0;
     virtual void cursor_image_removed() = 0;
     virtual void placed_relative(geometry::Rectangle const& placement) = 0;
+    virtual void input_consumed(MirEvent const* event) = 0;
+    virtual void start_drag_and_drop(std::vector<uint8_t> const& handle) = 0;
 
 protected:
     SurfaceObserver() = default;
