@@ -2,7 +2,7 @@
  * Copyright © 2013 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -33,8 +33,7 @@ class NullDisplayBuffer : public graphics::DisplayBuffer, public graphics::Nativ
 public:
     geometry::Rectangle view_area() const override { return geometry::Rectangle(); }
     bool overlay(graphics::RenderableList const&) override { return false; }
-    MirOrientation orientation() const override { return mir_orientation_normal; }
-    MirMirrorMode mirror_mode() const override { return mir_mirror_mode_none; }
+    glm::mat2 transformation() const override { return {}; }
     NativeDisplayBuffer* native_display_buffer() override { return this; }
 };
 
