@@ -2,7 +2,7 @@
  * Copyright © 2013-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -156,6 +156,18 @@ MATCHER_P(KeyOfSymbol, keysym, "")
     return true;
 }
 
+MATCHER_P(KeyWithText, text, "")
+{
+    auto kev = maybe_key_event(to_address(arg));
+    if (kev == nullptr)
+        return false;
+
+    if(strcmp(mir_keyboard_event_key_text(kev), text))
+        return false;
+
+    return true;
+}
+
 MATCHER_P(KeyOfScanCode, code, "")
 {
     auto kev = maybe_key_event(to_address(arg));
