@@ -2,7 +2,7 @@
  * Copyright © 2012-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,8 +18,9 @@
 
 #include "mir_protobuf_rpc_channel.h"
 #include "rpc_report.h"
+#include "mir/input/input_receiver_report.h"
 
-#include "../surface_map.h"
+#include "mir/client/surface_map.h"
 #include "../buffer.h"
 #include "../presentation_chain.h"
 #include "../buffer_factory.h"
@@ -34,12 +35,12 @@
 #include "mir/variable_length_array.h"
 #include "mir/events/event_builders.h"
 #include "mir/events/event_private.h"
-#include "mir/events/serialization.h"
 #include "mir/events/surface_placement_event.h"
 
 #include "mir_protobuf.pb.h"  // For Buffer frig
 #include "mir_protobuf_wire.pb.h"
 
+#include "mir/event_printer.h"
 #include <boost/bind.hpp>
 #include <boost/throw_exception.hpp>
 #include <endian.h>
@@ -61,12 +62,14 @@ mclr::MirProtobufRpcChannel::MirProtobufRpcChannel(
     std::shared_ptr<DisplayConfiguration> const& disp_config,
     std::shared_ptr<input::InputDevices> const& input_devices,
     std::shared_ptr<RpcReport> const& rpc_report,
+    std::shared_ptr<input::receiver::InputReceiverReport> const& input_report,
     std::shared_ptr<LifecycleControl> const& lifecycle_control,
     std::shared_ptr<PingHandler> const& ping_handler,
     std::shared_ptr<ErrorHandler> const& error_handler,
     std::shared_ptr<EventSink> const& event_sink) :
     rpc_report(rpc_report),
     pending_calls(rpc_report),
+    input_report(input_report),
     surface_map(surface_map),
     buffer_factory(buffer_factory),
     display_configuration(disp_config),
@@ -197,6 +200,7 @@ void mclr::MirProtobufRpcChannel::call_method(
     google::protobuf::MessageLite* response,
     google::protobuf::Closure* complete)
 {
+    std::lock_guard<std::mutex> lk(discard_mutex);
     if (discard)
     {
        /*
@@ -211,6 +215,9 @@ void mclr::MirProtobufRpcChannel::call_method(
        return;
     }
 
+    if (method_name == "disconnect")
+        discard = true;
+
     // Only send message when details saved for handling response
     std::vector<mir::Fd> fds;
     if (parameters->GetTypeName() == "mir.protobuf.BufferRequest")
@@ -244,6 +251,7 @@ void mclr::MirProtobufRpcChannel::call_method(
 
 void mclr::MirProtobufRpcChannel::discard_future_calls()
 {
+    std::unique_lock<decltype(discard_mutex)> lk(discard_mutex);
     discard = true;
 }
 
@@ -354,11 +362,19 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
                         buffer->received();
                         break;
                     case mp::BufferOperation::update:
-                        map->buffer(buffer_id)->received(
-                            *mcl::protobuf_to_native_buffer(seq.buffer_request().buffer()));
+                        buffer = map->buffer(buffer_id);
+                        if (buffer)
+                        {
+                            buffer->received(
+                                *mcl::protobuf_to_native_buffer(seq.buffer_request().buffer()));
+                        }
                         break;
                     case mp::BufferOperation::remove:
-                        map->erase(buffer_id);
+                        /* The server never sends us an unsolicited ::remove request
+                         * (and clients have no way of dealing with one)
+                         *
+                         * Just ignore it, because we've already deleted our buffer.
+                         */
                         break;
                     default:
                         BOOST_THROW_EXCEPTION(std::runtime_error("unknown buffer operation"));
@@ -390,46 +406,56 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
             // But that's a job for later...
             try
             {
-                auto e = mev::deserialize_event(event.raw());
+                auto e = MirEvent::deserialize(event.raw());
                 if (e)
                 {
                     rpc_report->event_parsing_succeeded(*e);
 
-                    int surface_id = 0;
-                    bool is_surface_event = true;
+                    int window_id = 0;
+                    bool is_window_event = true;
 
                     switch (e->type())
                     {
                     case mir_event_type_window:
-                        surface_id = e->to_surface()->id();
+                        window_id = e->to_surface()->id();
                         break;
                     case mir_event_type_resize:
-                        surface_id = e->to_resize()->surface_id();
+                        window_id = e->to_resize()->surface_id();
                         break;
                     case mir_event_type_orientation:
-                        surface_id = e->to_orientation()->surface_id();
+                        window_id = e->to_orientation()->surface_id();
                         break;
                     case mir_event_type_close_window:
-                        surface_id = e->to_close_window()->surface_id();
+                        window_id = e->to_close_window()->surface_id();
                         break;
                     case mir_event_type_keymap:
-                        surface_id = e->to_keymap()->surface_id();
+                        input_report->received_event(*e);
+                        window_id = e->to_keymap()->surface_id();
                         break;
                     case mir_event_type_window_output:
-                        surface_id = e->to_window_output()->surface_id();
+                        window_id = e->to_window_output()->surface_id();
                         break;
                     case mir_event_type_window_placement:
-                        surface_id = e->to_window_placement()->id();
+                        window_id = e->to_window_placement()->id();
+                        break;
+                    case mir_event_type_input:
+                        input_report->received_event(*e);
+                        window_id = e->to_input()->window_id();
+                        break;
+                    case mir_event_type_input_device_state:
+                        input_report->received_event(*e);
+                        window_id = e->to_input_device_state()->window_id();
                         break;
                     default:
-                        is_surface_event = false;
+                        is_window_event = false;
                         event_sink->handle_event(*e);
                     }
 
-                    if (is_surface_event)
+                    if (is_window_event)
                         if (auto map = surface_map.lock())
-                            if (auto surf = map->surface(mf::SurfaceId(surface_id)))
+                            if (auto surf = map->surface(mf::SurfaceId(window_id)))
                                 surf->handle_event(*e);
+
                 }
             }
             catch(...)
