@@ -2,7 +2,7 @@
  * Copyright © 2012 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License version 3,
+ * under the terms of the GNU Lesser General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -112,3 +112,8 @@ void mgm::GBMBuffer::bind()
 {
     gl_bind_to_texture();
 }
+
+void mgm::GBMBuffer::bind_for_write()
+{
+    bind();
+}
