@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -36,17 +36,13 @@ namespace frontend { class ClientBuffers; }
 namespace compositor
 {
 class Schedule;
-class FrameDroppingPolicyFactory;
-class FrameDroppingPolicy;
 class Stream : public BufferStream
 {
 public:
-    Stream(
-        FrameDroppingPolicyFactory const& policy_factory,
-        std::shared_ptr<frontend::ClientBuffers>, geometry::Size sz, MirPixelFormat format);
+    Stream(geometry::Size sz, MirPixelFormat format);
+    ~Stream();
 
-    void swap_buffers(
-        graphics::Buffer* old_buffer, std::function<void(graphics::Buffer* new_buffer)> complete) override;
+    void submit_buffer(std::shared_ptr<graphics::Buffer> const& buffer) override;
     void with_most_recent_buffer_do(std::function<void(graphics::Buffer&)> const& exec) override;
     MirPixelFormat pixel_format() const override;
     void add_observer(std::shared_ptr<scene::SurfaceObserver> const& observer) override;
@@ -56,42 +52,25 @@ public:
     geometry::Size stream_size() override;
     void resize(geometry::Size const& size) override;
     void allow_framedropping(bool) override;
-    void drop_outstanding_requests() override;
+    bool framedropping() const override;
     int buffers_ready_for_compositor(void const* user_id) const override;
     void drop_old_buffers() override;
     bool has_submitted_buffer() const override;
-    void associate_buffer(graphics::BufferID) override;
-    void disassociate_buffer(graphics::BufferID) override;
     void set_scale(float scale) override;
 
 private:
     enum class ScheduleMode;
-    struct DroppingCallback : mir::LockableCallback
-    {
-        DroppingCallback(Stream* stream);
-        void operator()() override;
-        void lock() override;
-        void unlock() override;
-        Stream* stream;
-        std::unique_lock<std::mutex> guard_lock;
-    };
     void transition_schedule(std::shared_ptr<Schedule>&& new_schedule, std::lock_guard<std::mutex> const&);
-    void drop_frame();
 
     std::mutex mutable mutex;
-    std::unique_ptr<compositor::FrameDroppingPolicy> drop_policy;
     ScheduleMode schedule_mode;
     std::shared_ptr<Schedule> schedule;
-    std::shared_ptr<frontend::ClientBuffers> buffers;
     std::shared_ptr<MultiMonitorArbiter> const arbiter;
     geometry::Size size; 
-    MirPixelFormat const pf;
+    MirPixelFormat pf;
     bool first_frame_posted;
 
     scene::SurfaceObservers observers;
-
-    std::set<graphics::BufferID> associated_buffers;
-    unsigned int client_owned_buffer_count(std::lock_guard<decltype(mutex)> const&) const;
 };
 }
 }
