@@ -2,7 +2,7 @@
  * Copyright © 2012-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -47,6 +47,7 @@
 #include "mir/frontend/screencast.h"
 #include "mir/frontend/prompt_session.h"
 #include "mir/frontend/buffer_stream.h"
+#include "mir/frontend/input_configuration_changer.h"
 #include "mir/input/input_device_hub.h"
 #include "mir/input/mir_input_config.h"
 #include "mir/input/mir_input_config_serialization.h"
@@ -58,12 +59,15 @@
 #include "mir/fd.h"
 #include "mir/cookie/authority.h"
 #include "mir/module_properties.h"
+#include "mir/graphics/graphic_buffer_allocator.h"
+#include "mir/executor.h"
 
 #include "mir/geometry/rectangles.h"
 #include "protobuf_buffer_packer.h"
 #include "protobuf_input_converter.h"
 
 #include "mir_toolkit/client_types.h"
+#include "mir_toolkit/cursors.h"
 
 #include <boost/exception/get_error_info.hpp>
 #include <boost/exception/errinfo_errno.hpp>
@@ -84,13 +88,6 @@ namespace geom = mir::geometry;
 
 namespace
 {
-//TODO: accept other pixel format types
-void throw_if_unsuitable_for_cursor(mf::BufferStream& stream)
-{
-    if (stream.pixel_format() != mir_pixel_format_argb_8888)
-        BOOST_THROW_EXCEPTION(std::logic_error("Only argb8888 buffer streams may currently be attached to the cursor"));
-}
-
 mg::GammaCurve convert_string_to_gamma_curve(std::string const& str_bytes)
 {
     mg::GammaCurve out(str_bytes.size() / (sizeof(mg::GammaCurve::value_type) / sizeof(char)));
@@ -114,7 +111,10 @@ mf::SessionMediator::SessionMediator(
     std::shared_ptr<scene::CoordinateTranslator> const& translator,
     std::shared_ptr<scene::ApplicationNotRespondingDetector> const& anr_detector,
     std::shared_ptr<mir::cookie::Authority> const& cookie_authority,
-    std::shared_ptr<mir::input::InputDeviceHub> const& hub) :
+    std::shared_ptr<mf::InputConfigurationChanger> const& input_changer,
+    std::vector<mir::ExtensionDescription> const& extensions,
+    std::shared_ptr<mg::GraphicBufferAllocator> const& allocator,
+    mir::Executor& executor) :
     client_pid_(0),
     shell(shell),
     ipc_operations(ipc_operations),
@@ -131,7 +131,10 @@ mf::SessionMediator::SessionMediator(
     translator{translator},
     anr_detector{anr_detector},
     cookie_authority(cookie_authority),
-    hub(hub)
+    input_changer(input_changer),
+    extensions(extensions),
+    allocator{allocator},
+    executor{executor}
 {
 }
 
@@ -185,33 +188,22 @@ void mf::SessionMediator::connect(
     auto protobuf_config = response->mutable_display_configuration();
     mfd::pack_protobuf_display_configuration(*protobuf_config, *display_config);
 
-    MirInputConfig temp;
-    hub->for_each_input_device(
-        [&temp](auto const& dev)
-        {
-            MirInputDevice conf(dev.id(), dev.capabilities(), dev.name(), dev.unique_id());
-            auto ptr_conf = dev.pointer_configuration();
-            auto tpd_conf = dev.touchpad_configuration();
-            auto kbd_conf = dev.keyboard_configuration();
-
-            if (ptr_conf.is_set())
-                conf.set_pointer_config(ptr_conf.value());
-            if (tpd_conf.is_set())
-                conf.set_touchpad_config(tpd_conf.value());
-            if (kbd_conf.is_set())
-                conf.set_keyboard_config(kbd_conf.value());
-
-            temp.add_device_config(conf);
-        });
-
-    response->set_input_configuration(mi::serialize_input_config(temp));
+    response->set_input_configuration(mi::serialize_input_config(input_changer->base_configuration()));
 
     for (auto pf : surface_pixel_formats)
         response->add_surface_pixel_format(static_cast<::google::protobuf::uint32>(pf));
 
     resource_cache->save_resource(response, ipc_package);
 
-    response->set_coordinate_translation_present(translator->translation_supported());
+    for ( auto const& ext : extensions )
+    {
+        if (ext.version.empty()) //malformed plugin, ignore
+            continue;
+        auto e = response->add_extension();
+        e->set_name(ext.name);
+        for(auto const& v : ext.version)
+            e->add_version(v);
+    }
 
     done->Run();
 }
@@ -367,9 +359,6 @@ void mf::SessionMediator::create_surface(
     response->set_pixel_format(request->pixel_format());
     response->set_buffer_usage(request->buffer_usage());
 
-    if (surface->supports_input())
-        response->add_fd(surface->client_input_fd());
-    
     for (unsigned int i = 0; i < mir_window_attribs; i++)
     {
         auto setting = response->add_attributes();
@@ -391,6 +380,63 @@ void mf::SessionMediator::create_surface(
     buffering_sender->uncork();
 }
 
+namespace
+{
+    class AutoSendBuffer : public mg::Buffer
+    {
+    public:
+        AutoSendBuffer(
+            std::shared_ptr<mg::Buffer> const& wrapped,
+            mir::Executor& executor,
+            std::weak_ptr<mf::BufferSink> const& sink)
+            : buffer{wrapped},
+              executor{executor},
+              sink{sink}
+        {
+        }
+        ~AutoSendBuffer()
+        {
+            executor.spawn(
+                [maybe_sink = sink, to_send = std::move(buffer)]()
+                {
+                    if (auto const live_sink = maybe_sink.lock())
+                        live_sink->update_buffer(*to_send);
+                });
+        }
+
+        std::shared_ptr<mir::graphics::NativeBuffer> native_buffer_handle() const override
+        {
+            return buffer->native_buffer_handle();
+        }
+
+        mir::graphics::BufferID id() const override
+        {
+            return buffer->id();
+        }
+
+        mir::geometry::Size size() const override
+        {
+            return buffer->size();
+        }
+
+        MirPixelFormat pixel_format() const override
+        {
+            return buffer->pixel_format();
+        }
+
+        mir::graphics::NativeBufferBase *native_buffer_base() override
+        {
+            return buffer->native_buffer_base();
+        }
+
+    private:
+        std::shared_ptr<mg::Buffer> buffer;
+        mir::Executor& executor;
+        std::weak_ptr<mf::BufferSink> const sink;
+    };
+
+}
+
 void mf::SessionMediator::submit_buffer(
     mir::protobuf::BufferRequest const* request,
     mir::protobuf::Void*,
@@ -405,15 +451,37 @@ void mf::SessionMediator::submit_buffer(
     auto stream = session->get_buffer_stream(stream_id);
 
     mfd::ProtobufBufferPacker request_msg{const_cast<mir::protobuf::Buffer*>(&request->buffer())};
-    auto b = session->get_buffer(buffer_id);
+    auto b = buffer_cache.at(buffer_id);
     ipc_operations->unpack_buffer(request_msg, *b);
 
-    stream->submit_buffer(b);
+    stream->submit_buffer(std::make_shared<AutoSendBuffer>(b, executor, event_sink));
 
     done->Run();
 }
 
-void mf::SessionMediator::allocate_buffers( 
+namespace
+{
+bool validate_buffer_request(mir::protobuf::BufferStreamParameters const& req)
+{
+    // A valid buffer request has either flags & native_format set
+    // or buffer_usage & pixel_format set, not both.
+    return
+        (
+            req.has_pixel_format() &&
+            req.has_buffer_usage() &&
+            !req.has_native_format() &&
+            !req.has_flags()
+        ) ||
+        (
+            !req.has_pixel_format() &&
+            !req.has_buffer_usage() &&
+            req.has_native_format() &&
+            req.has_flags()
+        );
+}
+}
+
+void mf::SessionMediator::allocate_buffers(
     mir::protobuf::BufferAllocation const* request,
     mir::protobuf::Void*,
     google::protobuf::Closure* done)
@@ -426,38 +494,56 @@ void mf::SessionMediator::allocate_buffers(
     for (auto i = 0; i < request->buffer_requests().size(); i++)
     {
         auto const& req = request->buffer_requests(i);
-
-        mg::BufferID id;
-        if (req.has_flags() && req.has_native_format())
+        std::shared_ptr<mg::Buffer> buffer;
+        try
         {
-            auto session_ext = std::dynamic_pointer_cast<SessionExtensions>(session);
-            id = session_ext->create_buffer({req.width(), req.height()}, req.native_format(), req.flags());
-        }
-        else if (req.has_buffer_usage() && req.has_pixel_format())
-        {
-            auto const usage = static_cast<mg::BufferUsage>(req.buffer_usage());
-            geom::Size const size{req.width(), req.height()};
-            auto const pf = static_cast<MirPixelFormat>(req.pixel_format());
-            if (usage == mg::BufferUsage::software)
+            if (!validate_buffer_request(req))
             {
-                auto session_ext = std::dynamic_pointer_cast<SessionExtensions>(session);
-                id = session_ext->create_buffer(size, pf);
+                BOOST_THROW_EXCEPTION(std::logic_error("Invalid buffer request"));
+            }
+
+            if (req.has_flags() && req.has_native_format())
+            {
+                buffer = allocator->alloc_buffer(
+                    {req.width(), req.height()},
+                    req.native_format(),
+                    req.flags());
             }
             else
             {
-                //legacy route, server-selected pf and usage
-                id = session->create_buffer(mg::BufferProperties{size, pf, mg::BufferUsage::hardware});
+                auto const usage = static_cast<mg::BufferUsage>(req.buffer_usage());
+                geom::Size const size{req.width(), req.height()};
+                auto const pf = static_cast<MirPixelFormat>(req.pixel_format());
+                if (usage == mg::BufferUsage::software)
+                {
+                    buffer = allocator->alloc_software_buffer(size, pf);
+                }
+                else
+                {
+                    //legacy route, server-selected pf and usage
+                    buffer =
+                        allocator->alloc_buffer(mg::BufferProperties{size, pf, mg::BufferUsage::hardware});
+                }
             }
-        }
-        else
-        {
-            BOOST_THROW_EXCEPTION(std::logic_error("Invalid buffer request"));
-        }
 
-        if (request->has_id())
+            if (request->has_id())
+            {
+                auto const stream_id = mf::BufferStreamId{request->id().value()};
+                // We don't need the stream, but we *do* need to know it exists
+                auto stream = session->get_buffer_stream(stream_id);
+                stream_associated_buffers.insert(std::make_pair(stream_id, buffer->id()));
+            }
+
+            // TODO: Throw if insert fails (duplicate ID)?
+            buffer_cache.insert(std::make_pair(buffer->id(), buffer));
+            event_sink->add_buffer(*buffer);
+        }
+        catch (std::exception const& err)
         {
-            auto stream = session->get_buffer_stream(mf::BufferStreamId(request->id().value()));
-            stream->associate_buffer(id);
+            event_sink->error_buffer(
+                geom::Size{req.width(), req.height()},
+                static_cast<MirPixelFormat>(req.pixel_format()),
+                err.what());
         }
     }
     done->Run();
@@ -473,26 +559,35 @@ void mf::SessionMediator::release_buffers(
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
     observer->session_release_buffers_called(session->name());
+
+    std::vector<mg::BufferID> to_release(request->buffers().size());
+    std::transform(
+        request->buffers().begin(),
+        request->buffers().end(),
+        to_release.begin(),
+        [](auto buffer)
+        {
+            return mg::BufferID{static_cast<uint32_t>(buffer.buffer_id())};
+        });
+
     if (request->has_id())
     {
-        auto stream_id = mf::BufferStreamId{request->id().value()};
-        try
+        auto const stream_id = mf::BufferStreamId{request->id().value()};
+
+        auto const associated_range = stream_associated_buffers.equal_range(stream_id);
+        for (auto match = associated_range.first; match != associated_range.second;)
         {
-            auto stream = session->get_buffer_stream(stream_id);
-            for (auto i = 0; i < request->buffers().size(); i++)
+            auto const current = match;
+            ++match;
+            if (std::find(to_release.begin(), to_release.end(), current->second) != to_release.end())
             {
-                mg::BufferID buffer_id{static_cast<uint32_t>(request->buffers(i).buffer_id())};
-                stream->disassociate_buffer(buffer_id);
+                stream_associated_buffers.erase(current);
             }
         }
-        catch(...)
-        {
-        }
     }
-    for (auto i = 0; i < request->buffers().size(); i++)
+    for (auto const& buffer_id : to_release)
     {
-        mg::BufferID buffer_id{static_cast<uint32_t>(request->buffers(i).buffer_id())};
-        session->destroy_buffer(buffer_id);
+        buffer_cache.erase(buffer_id);
     }
    done->Run();
 }
@@ -665,7 +760,14 @@ void mf::SessionMediator::modify_surface(
 
     if (surface_specification.has_cursor_name())
     {
-        mods.cursor_image = cursor_images->image(surface_specification.cursor_name(), mi::default_cursor_size);
+        if (surface_specification.cursor_name() == mir_disabled_cursor_name)
+        {
+            mods.cursor_image = nullptr;
+        }
+        else
+        {
+            mods.cursor_image = cursor_images->image(surface_specification.cursor_name(), mi::default_cursor_size);
+        }
     }
 
     if (surface_specification.has_cursor_id() &&
@@ -673,9 +775,9 @@ void mf::SessionMediator::modify_surface(
         surface_specification.has_hotspot_y())
     {
         mf::BufferStreamId id{surface_specification.cursor_id().value()};
-        throw_if_unsuitable_for_cursor(*session->get_buffer_stream(id));
+        auto stream = session->get_buffer_stream(id);
         mods.stream_cursor = msh::StreamCursor{
-            id, geom::Displacement{surface_specification.hotspot_x(), surface_specification.hotspot_y()} }; 
+            id, geom::Displacement{surface_specification.hotspot_x(), surface_specification.hotspot_y()} };
     }
 
     if (surface_specification.input_shape_size() > 0)
@@ -836,6 +938,7 @@ void mf::SessionMediator::create_screencast(
     protobuf_screencast->mutable_buffer_stream()->set_pixel_format(pixel_format);
     protobuf_screencast->mutable_buffer_stream()->mutable_id()->set_value(
         screencast_session_id.as_value());
+    protobuf_screencast->mutable_buffer_stream()->set_pixel_format(pixel_format);
 
     done->Run();
 }
@@ -878,8 +981,7 @@ void mf::SessionMediator::screencast_to_buffer(
 {
     auto session = weak_session.lock();
     ScreencastSessionId const screencast_session_id{request->id().value()};
-    auto buffer = session->get_buffer(mg::BufferID{request->buffer_id()});
-    mf::ScreencastSessionId const screencast_id{request->id().value()};
+    auto buffer = buffer_cache.at(mg::BufferID{request->buffer_id()});
     screencast->capture(screencast_session_id, buffer);
     done->Run();
 }
@@ -931,6 +1033,13 @@ void mf::SessionMediator::release_buffer_stream(
 
     session->destroy_buffer_stream(id);
 
+    auto const associated_range = stream_associated_buffers.equal_range(id) ;
+    for (auto match = associated_range.first; match != associated_range.second; ++match)
+    {
+        buffer_cache.erase(match->second);
+    }
+    stream_associated_buffers.erase(id);
+
     done->Run();
 }
 
@@ -974,8 +1083,6 @@ void mf::SessionMediator::configure_cursor(
         auto hotspot = geom::Displacement{cursor_request->hotspot_x(), cursor_request->hotspot_y()};
         auto stream = session->get_buffer_stream(stream_id);
 
-        throw_if_unsuitable_for_cursor(*stream);
-
         surface->set_cursor_stream(stream, hotspot);
     }
     else
@@ -1175,16 +1282,15 @@ void mf::SessionMediator::configure_buffer_stream(
     done->Run();
 }
 
-void mf::SessionMediator::raise_surface(
-    mir::protobuf::RaiseRequest const* request,
-    mir::protobuf::Void*,
-    google::protobuf::Closure* done)
+void mir::frontend::SessionMediator::request_operation(
+    mir::protobuf::RequestWithAuthority const* request,
+    mir::protobuf::Void*, google::protobuf::Closure* done)
 {
     auto const session = weak_session.lock();
     if (!session)
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
 
-    auto const cookie     = request->cookie();
+    auto const cookie     = request->authority();
     auto const surface_id = request->surface_id();
 
     auto cookie_string = cookie.cookie();
@@ -1192,7 +1298,62 @@ void mf::SessionMediator::raise_surface(
     std::vector<uint8_t> cookie_bytes(cookie_string.begin(), cookie_string.end());
     auto const cookie_ptr = cookie_authority->make_cookie(cookie_bytes);
 
-    shell->raise_surface(session, mf::SurfaceId{surface_id.value()}, cookie_ptr->timestamp());
+    switch (request->operation())
+    {
+    case mir::protobuf::RequestOperation::START_DRAG_AND_DROP:
+        shell->request_operation(
+            session, mf::SurfaceId{surface_id.value()},
+            cookie_ptr->timestamp(),
+            Shell::UserRequest::drag_and_drop);
+        break;
+
+    case mir::protobuf::RequestOperation::MAKE_ACTIVE:
+        shell->request_operation(
+            session, mf::SurfaceId{surface_id.value()},
+            cookie_ptr->timestamp(),
+            Shell::UserRequest::activate);
+        break;
+
+    case mir::protobuf::RequestOperation::USER_MOVE:
+        shell->request_operation(
+            session, mf::SurfaceId{surface_id.value()},
+            cookie_ptr->timestamp(),
+            Shell::UserRequest::move);
+        break;
+
+    default:
+        break;
+    }
+
+    done->Run();
+}
+
+void mf::SessionMediator::apply_input_configuration(
+    mir::protobuf::InputConfigurationRequest const* request,
+    mir::protobuf::Void*,
+    google::protobuf::Closure* done)
+{
+    auto const session = weak_session.lock();
+    if (!session)
+        BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
+
+    auto conf = mi::deserialize_input_config(request->input_configuration());
+    input_changer->configure(session, std::move(conf));
+
+    done->Run();
+}
+
+void mf::SessionMediator::set_base_input_configuration(
+    mir::protobuf::InputConfigurationRequest const* request,
+    mir::protobuf::Void*,
+    google::protobuf::Closure* done)
+{
+    auto const session = weak_session.lock();
+    if (!session)
+        BOOST_THROW_EXCEPTION(std::logic_error("Invalid application session"));
+
+    auto conf = mi::deserialize_input_config(request->input_configuration());
+    input_changer->set_base_configuration(std::move(conf));
 
     done->Run();
 }
@@ -1228,6 +1389,20 @@ mf::SessionMediator::unpack_and_sanitize_display_configuration(
         dest.gamma = {convert_string_to_gamma_curve(src.gamma_red()),
                       convert_string_to_gamma_curve(src.gamma_green()),
                       convert_string_to_gamma_curve(src.gamma_blue())};
+
+        if (src.has_custom_logical_size())
+        {
+            if (!src.custom_logical_size())
+            {   // User has explicitly removed logical size customization
+                if (dest.custom_logical_size.is_set())
+                    (void)dest.custom_logical_size.consume();
+            }
+            else if (src.has_logical_width() && src.has_logical_height())
+            {   // User has explicitly set a logical size customization
+                dest.custom_logical_size = {src.logical_width(),
+                                            src.logical_height()};
+            }
+        }
     });
 
     return config;
@@ -1246,7 +1421,6 @@ void mf::SessionMediator::destroy_screencast_sessions()
         screencast_buffer_tracker.remove_session(id);
 }
 
-
 auto mf::detail::PromptSessionStore::insert(std::shared_ptr<PromptSession> const& session) -> PromptSessionId
 {
     std::lock_guard<decltype(mutex)> lock{mutex};
