@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -99,10 +99,8 @@ int mf::ShellWrapper::get_surface_attribute(
     return wrapped->get_surface_attribute(session, surface_id, attrib);
 }
 
-void mf::ShellWrapper::raise_surface(
-    std::shared_ptr<Session> const& session,
-    SurfaceId surface_id,
-    uint64_t timestamp)
+void mf::ShellWrapper::request_operation(
+    std::shared_ptr<Session> const& session, SurfaceId surface_id, uint64_t timestamp, UserRequest request)
 {
-    wrapped->raise_surface(session, surface_id, timestamp);
+    wrapped->request_operation(session, surface_id, timestamp, request);
 }
