@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -75,10 +75,11 @@ public:
         SurfaceId surface_id,
         MirWindowAttrib attrib) override;
 
-    void raise_surface(
+    void request_operation(
         std::shared_ptr<Session> const& session,
         SurfaceId surface_id,
-        uint64_t timestamp) override;
+        uint64_t timestamp,
+        UserRequest request) override;
 
 protected:
     std::shared_ptr<Shell> const wrapped;
