@@ -2,7 +2,7 @@
  * Copyright © 2013 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -117,17 +117,24 @@ void mgn::detail::DisplayBuffer::bind()
 bool mgn::detail::DisplayBuffer::overlay(RenderableList const& list)
 {
     if ((passthrough_option == mgn::PassthroughOption::disabled) ||
-        list.empty() ||
-        (list.back()->screen_position() != area) ||
-        (list.back()->alpha() != 1.0f) ||
-        (list.back()->shaped()) ||
-        (list.back()->transformation() != identity))
+        list.empty())
     {
         //could not represent scene with subsurfaces
         return false;
     }
 
-    auto passthrough_buffer = list.back()->buffer();
+    auto const topmost = list.back();
+
+    if ((topmost->screen_position() != area) ||
+        (topmost->alpha() != 1.0f) ||
+        (topmost->shaped()) ||
+        (topmost->transformation() != identity))
+    {
+        //could not represent scene with subsurfaces
+        return false;
+    }
+
+    auto passthrough_buffer = topmost->buffer();
     auto native = dynamic_cast<mgn::NativeBuffer*>(passthrough_buffer->native_buffer_handle().get());
     if (!native)
         return false;
@@ -144,7 +151,7 @@ bool mgn::detail::DisplayBuffer::overlay(RenderableList const& list)
         if ((submission_info == last_submitted) && (submitted != submitted_buffers.end()))
             return true;
 
-        if (list.back()->swap_interval() == 0)
+        if (topmost->swap_interval() == 0)
             host_chain->set_submission_mode(mgn::SubmissionMode::dropping);
         else
             host_chain->set_submission_mode(mgn::SubmissionMode::queueing);
@@ -161,7 +168,7 @@ bool mgn::detail::DisplayBuffer::overlay(RenderableList const& list)
     if (content != BackingContent::chain)
     {
         auto spec = host_connection->create_surface_spec();
-        spec->add_chain(*host_chain, geom::Displacement{0,0}, passthrough_buffer->size());
+        spec->add_chain(*host_chain, geom::Displacement{0,0}, area.size);
         content = BackingContent::chain;
         host_surface->apply_spec(*spec);
     }
@@ -176,18 +183,9 @@ void mgn::detail::DisplayBuffer::release_buffer(MirBuffer* b, MirPresentationCha
         submitted_buffers.erase(buf);
 }
 
-MirOrientation mgn::detail::DisplayBuffer::orientation() const
+glm::mat2 mgn::detail::DisplayBuffer::transformation() const
 {
-    /*
-     * Always normal orientation. The real rotation is handled by the
-     * native display.
-     */
-    return mir_orientation_normal;
-}
-
-MirMirrorMode mgn::detail::DisplayBuffer::mirror_mode() const
-{
-    return mir_mirror_mode_none;
+    return {};
 }
 
 mgn::detail::DisplayBuffer::~DisplayBuffer() noexcept
