@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,8 +25,11 @@
 #include "mir/input/input_sink.h"
 #include "mir/input/seat.h"
 #include "mir/input/input_device_hub.h"
+#include "mir/input/input_device_observer.h"
 #include "mir/input/input_device_info.h"
 #include "mir/input/mir_input_config.h"
+#include "mir/thread_safe_list.h"
+#include "mir/optional_value.h"
 
 #include "mir_toolkit/event.h"
 
@@ -39,10 +42,6 @@ namespace mir
 {
 class ServerActionQueue;
 class ServerStatusListener;
-namespace frontend
-{
-class EventSink;
-}
 namespace cookie
 {
 class Authority;
@@ -60,14 +59,32 @@ class InputDeviceObserver;
 class DefaultDevice;
 class Seat;
 class KeyMapper;
+class DefaultInputDeviceHub;
 
-class DefaultInputDeviceHub : public InputDeviceRegistry, public InputDeviceHub
+struct ExternalInputDeviceHub : InputDeviceHub
+{
+    ExternalInputDeviceHub(std::shared_ptr<InputDeviceHub> const& actual_hub,
+                           std::shared_ptr<ServerActionQueue> const& observer_queue);
+    ~ExternalInputDeviceHub();
+
+    void add_observer(std::shared_ptr<InputDeviceObserver> const&) override;
+    void remove_observer(std::weak_ptr<InputDeviceObserver> const&) override;
+    void for_each_input_device(std::function<void(Device const& device)> const& callback) override;
+    void for_each_mutable_input_device(std::function<void(Device& device)> const& callback) override;
+
+private:
+    struct Internal;
+    std::shared_ptr<Internal> data;
+    std::shared_ptr<InputDeviceHub> hub;
+};
+
+class DefaultInputDeviceHub :
+    public InputDeviceRegistry,
+    public InputDeviceHub
 {
 public:
-    DefaultInputDeviceHub(std::shared_ptr<frontend::EventSink> const& sink,
-                          std::shared_ptr<Seat> const& seat,
+    DefaultInputDeviceHub(std::shared_ptr<Seat> const& seat,
                           std::shared_ptr<dispatch::MultiplexingDispatchable> const& input_multiplexer,
-                          std::shared_ptr<ServerActionQueue> const& observer_queue,
                           std::shared_ptr<cookie::Authority> const& cookie_authority,
                           std::shared_ptr<KeyMapper> const& key_mapper,
                           std::shared_ptr<ServerStatusListener> const& server_status_listener);
@@ -81,21 +98,20 @@ public:
     void remove_observer(std::weak_ptr<InputDeviceObserver> const&) override;
     void for_each_input_device(std::function<void(Device const& device)> const& callback) override;
     void for_each_mutable_input_device(std::function<void(Device& device)> const& callback) override;
-
-
 private:
-    void update_spots();
     void add_device_handle(std::shared_ptr<DefaultDevice> const& handle);
     void remove_device_handle(MirInputDeviceId id);
     void device_changed(Device* dev);
     void emit_changed_devices();
     MirInputDeviceId create_new_device_id();
+    void store_device_config(DefaultDevice const& dev);
+    std::shared_ptr<DefaultDevice> restore_or_create_device(InputDevice& dev,
+                                                            std::shared_ptr<dispatch::ActionQueue> const& queue);
+    mir::optional_value<MirInputDevice> get_stored_device_config(std::string const& id);
 
     std::shared_ptr<Seat> const seat;
-    std::shared_ptr<frontend::EventSink> const sink;
     std::shared_ptr<dispatch::MultiplexingDispatchable> const input_dispatchable;
-    std::mutex observer_guard;
-    std::shared_ptr<ServerActionQueue> const observer_queue;
+    std::mutex mutable handles_guard;
     std::shared_ptr<dispatch::ActionQueue> const device_queue;
     std::shared_ptr<cookie::Authority> const cookie_authority;
     std::shared_ptr<KeyMapper> const key_mapper;
@@ -110,7 +126,8 @@ private:
                          std::shared_ptr<cookie::Authority> const& cookie_authority,
                          std::shared_ptr<DefaultDevice> const& handle);
         void handle_input(MirEvent& event) override;
-        mir::geometry::Rectangle bounding_rectangle() const override;
+        geometry::Rectangle bounding_rectangle() const override;
+        input::OutputInfo output_info(uint32_t output_id) const override;
         bool device_matches(std::shared_ptr<InputDevice> const& dev) const;
         void start(std::shared_ptr<Seat> const& seat, std::shared_ptr<dispatch::MultiplexingDispatchable> const& dispatchable);
         void stop(std::shared_ptr<dispatch::MultiplexingDispatchable> const& dispatchable);
@@ -131,10 +148,13 @@ private:
     std::vector<std::shared_ptr<Device>> handles;
     MirInputConfig config;
     std::vector<std::unique_ptr<RegisteredDevice>> devices;
-    std::vector<std::shared_ptr<InputDeviceObserver>> observers;
+    ThreadSafeList<std::shared_ptr<InputDeviceObserver>> observers;
     std::mutex changed_devices_guard;
     std::unique_ptr<std::vector<std::shared_ptr<Device>>> changed_devices;
 
+    std::mutex stored_configurations_guard;
+    std::vector<MirInputDevice> stored_devices;
+
     MirInputDeviceId device_id_generator;
     bool ready{false};
 };
