@@ -2,7 +2,7 @@
  * Copyright © 2012-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,7 +20,6 @@
 #include "snapshot_strategy.h"
 #include "default_session_container.h"
 #include "output_properties_cache.h"
-#include "../compositor/buffer_map.h"
 
 #include "mir/scene/surface.h"
 #include "mir/scene/surface_event_source.h"
@@ -32,6 +31,7 @@
 #include "mir/compositor/buffer_stream.h"
 #include "mir/events/event_builders.h"
 #include "mir/frontend/event_sink.h"
+#include "mir/graphics/graphic_buffer_allocator.h"
 
 #include <boost/throw_exception.hpp>
 
@@ -58,7 +58,7 @@ ms::ApplicationSession::ApplicationSession(
     std::shared_ptr<SessionListener> const& session_listener,
     mg::DisplayConfiguration const& initial_config,
     std::shared_ptr<mf::EventSink> const& sink,
-    std::shared_ptr<graphics::GraphicBufferAllocator> const&) : 
+    std::shared_ptr<graphics::GraphicBufferAllocator> const& gralloc) : 
     surface_stack(surface_stack),
     surface_factory(surface_factory),
     buffer_stream_factory(buffer_stream_factory),
@@ -67,7 +67,7 @@ ms::ApplicationSession::ApplicationSession(
     snapshot_strategy(snapshot_strategy),
     session_listener(session_listener),
     event_sink(sink),
-    buffers(buffer_stream_factory->create_buffer_map(sink)),
+    gralloc(gralloc),
     next_surface_id(0)
 {
     assert(surface_stack);
@@ -140,11 +140,11 @@ mf::SurfaceId ms::ApplicationSession::create_surface(
     surface_stack->add_surface(surface, params.input_mode);
 
     if (params.state.is_set())
-        surface->configure(mir_surface_attrib_state, params.state.value());
+        surface->configure(mir_window_attrib_state, params.state.value());
     if (params.type.is_set())
-        surface->configure(mir_surface_attrib_type, params.type.value());
+        surface->configure(mir_window_attrib_type, params.type.value());
     if (params.preferred_orientation.is_set())
-        surface->configure(mir_surface_attrib_preferred_orientation, params.preferred_orientation.value());
+        surface->configure(mir_window_attrib_preferred_orientation, params.preferred_orientation.value());
     if (params.input_shape.is_set())
         surface->set_input_region(params.input_shape.value());
 
@@ -211,17 +211,17 @@ std::shared_ptr<ms::Surface> ms::ApplicationSession::surface_after(std::shared_p
         {
             switch (s.second->type())
             {
-            case mir_surface_type_normal:       /**< AKA "regular"                       */
-            case mir_surface_type_utility:      /**< AKA "floating"                      */
-            case mir_surface_type_dialog:
-            case mir_surface_type_satellite:    /**< AKA "toolbox"/"toolbar"             */
-            case mir_surface_type_freestyle:
-            case mir_surface_type_menu:
-            case mir_surface_type_inputmethod:  /**< AKA "OSK" or handwriting etc.       */
+            case mir_window_type_normal:       /**< AKA "regular"                       */
+            case mir_window_type_utility:      /**< AKA "floating"                      */
+            case mir_window_type_dialog:
+            case mir_window_type_satellite:    /**< AKA "toolbox"/"toolbar"             */
+            case mir_window_type_freestyle:
+            case mir_window_type_menu:
+            case mir_window_type_inputmethod:  /**< AKA "OSK" or handwriting etc.       */
                 return true;
 
-            case mir_surface_type_gloss:
-            case mir_surface_type_tip:          /**< AKA "tooltip"                       */
+            case mir_window_type_gloss:
+            case mir_window_type_tip:          /**< AKA "tooltip"                       */
             default:
                 // Cannot have input focus - skip it
                 return false;
@@ -233,6 +233,9 @@ std::shared_ptr<ms::Surface> ms::ApplicationSession::surface_after(std::shared_p
     if (next == surfaces.end())
         next = std::find_if(begin(surfaces), current, can_take_focus);
 
+    if (next == end(surfaces))
+        return {};
+
     return next->second;
 }
 
@@ -281,15 +284,6 @@ pid_t ms::ApplicationSession::process_id() const
     return pid;
 }
 
-void ms::ApplicationSession::drop_outstanding_requests()
-{
-    std::unique_lock<std::mutex> lock(surfaces_and_streams_mutex);
-    for (auto& stream : streams)
-    {
-        stream.second->drop_outstanding_requests();
-    }
-}
-
 void ms::ApplicationSession::hide()
 {
     std::unique_lock<std::mutex> lock(surfaces_and_streams_mutex);
@@ -328,6 +322,7 @@ void ms::ApplicationSession::send_display_config(mg::DisplayConfiguration const&
                     surface.first,
                     output_properties->dpi,
                     output_properties->scale,
+                    output_properties->refresh_rate,
                     output_properties->form_factor,
                     static_cast<uint32_t>(output_properties->id.as_value())
                     ));
@@ -335,9 +330,9 @@ void ms::ApplicationSession::send_display_config(mg::DisplayConfiguration const&
     }
 }
 
-void ms::ApplicationSession::send_input_device_change(std::vector<std::shared_ptr<mir::input::Device>> const& devices)
+void ms::ApplicationSession::send_input_config(MirInputConfig const& config)
 {
-    event_sink->handle_input_device_change(devices);
+    event_sink->handle_input_config_change(config);
 }
 
 void ms::ApplicationSession::set_lifecycle_state(MirLifecycleState state)
@@ -375,7 +370,7 @@ std::shared_ptr<mf::BufferStream> ms::ApplicationSession::get_buffer_stream(mf::
 mf::BufferStreamId ms::ApplicationSession::create_buffer_stream(mg::BufferProperties const& props)
 {
     auto const id = static_cast<mf::BufferStreamId>(next_id().as_value());
-    auto stream = buffer_stream_factory->create_buffer_stream(id, buffers, props);
+    auto stream = buffer_stream_factory->create_buffer_stream(id, props);
     
     std::unique_lock<std::mutex> lock(surfaces_and_streams_mutex);
     streams[id] = stream;
@@ -389,7 +384,6 @@ void ms::ApplicationSession::destroy_buffer_stream(mf::BufferStreamId id)
     if (stream_it == streams.end())
         BOOST_THROW_EXCEPTION(std::runtime_error("cannot destroy stream: Invalid BufferStreamId"));
 
-    stream_it->second->drop_outstanding_requests();
     streams.erase(stream_it);
 }
 
@@ -433,17 +427,7 @@ void ms::ApplicationSession::destroy_surface(std::unique_lock<std::mutex>& lock,
     surface_stack->remove_surface(surface);
 }
 
-mg::BufferID ms::ApplicationSession::create_buffer(mg::BufferProperties const& properties)
+void ms::ApplicationSession::send_error(mir::ClientVisibleError const& error)
 {
-    return buffers->add_buffer(properties);
-}
-
-void ms::ApplicationSession::destroy_buffer(mg::BufferID id)
-{
-    buffers->remove_buffer(id);
-}
-
-std::shared_ptr<mg::Buffer> ms::ApplicationSession::get_buffer(mg::BufferID id)
-{
-    return (*buffers)[id];
+    event_sink->handle_error(error);
 }
