@@ -2,7 +2,7 @@
  * Copyright © 2012-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,9 +21,7 @@
 #include "basic_surface.h"
 #include "mir/compositor/buffer_stream.h"
 #include "mir/frontend/event_sink.h"
-#include "mir/input/input_channel.h"
 #include "mir/shell/input_targeter.h"
-#include "mir/input/input_sender.h"
 #include "mir/graphics/buffer.h"
 #include "mir/graphics/cursor_image.h"
 #include "mir/graphics/pixel_format_utils.h"
@@ -140,6 +138,18 @@ void ms::SurfaceObservers::placed_relative(geometry::Rectangle const& placement)
                  { observer->placed_relative(placement); });
 }
 
+void ms::SurfaceObservers::input_consumed(MirEvent const* event)
+{
+    for_each([&](std::shared_ptr<SurfaceObserver> const& observer)
+                 { observer->input_consumed(event); });
+}
+
+void ms::SurfaceObservers::start_drag_and_drop(std::vector<uint8_t> const& handle)
+{
+    for_each([&](std::shared_ptr<SurfaceObserver> const& observer)
+                 { observer->start_drag_and_drop(handle); });
+}
+
 
 struct ms::CursorStreamImageAdapter
 {
@@ -229,8 +239,6 @@ ms::BasicSurface::BasicSurface(
     std::weak_ptr<Surface> const& parent,
     MirPointerConfinementState state,
     std::list<StreamInfo> const& layers,
-    std::shared_ptr<mi::InputChannel> const& input_channel,
-    std::shared_ptr<input::InputSender> const& input_sender,
     std::shared_ptr<mg::CursorImage> const& cursor_image,
     std::shared_ptr<SceneReport> const& report) :
     surface_name(name),
@@ -240,15 +248,12 @@ ms::BasicSurface::BasicSurface(
     input_mode(mi::InputReceptionMode::normal),
     custom_input_rectangles(),
     surface_buffer_stream(default_stream(layers)),
-    server_input_channel(input_channel),
-    input_sender(input_sender),
     cursor_image_(cursor_image),
     report(report),
     parent_(parent),
     layers(layers),
     confine_pointer_state_(state),
-    cursor_stream_adapter{std::make_unique<ms::CursorStreamImageAdapter>(*this)},
-    input_validator([this](MirEvent const& ev) { this->input_sender->send_event(ev, server_input_channel); })
+    cursor_stream_adapter{std::make_unique<ms::CursorStreamImageAdapter>(*this)}
 {
     report->surface_created(this, surface_name);
 }
@@ -258,12 +263,10 @@ ms::BasicSurface::BasicSurface(
     geometry::Rectangle rect,
     MirPointerConfinementState state,
     std::list<StreamInfo> const& layers,
-    std::shared_ptr<mi::InputChannel> const& input_channel,
-    std::shared_ptr<input::InputSender> const& input_sender,
     std::shared_ptr<mg::CursorImage> const& cursor_image,
     std::shared_ptr<SceneReport> const& report) :
     BasicSurface(name, rect, std::shared_ptr<Surface>{nullptr}, state, layers,
-                 input_channel, input_sender, cursor_image, report)
+                 cursor_image, report)
 {
 }
 
@@ -286,12 +289,6 @@ void ms::BasicSurface::move_to(geometry::Point const& top_left)
     observers.moved_to(top_left);
 }
 
-float ms::BasicSurface::alpha() const
-{
-    std::unique_lock<std::mutex> lk(guard);
-    return surface_alpha;
-}
-
 void ms::BasicSurface::set_hidden(bool hide)
 {
     {
@@ -318,25 +315,6 @@ std::shared_ptr<mf::BufferStream> ms::BasicSurface::primary_buffer_stream() cons
     return surface_buffer_stream;
 }
 
-bool ms::BasicSurface::supports_input() const
-{
-    if (server_input_channel  && server_input_channel->client_fd() != -1)
-        return true;
-    return false;
-}
-
-int ms::BasicSurface::client_input_fd() const
-{
-    if (!supports_input())
-        BOOST_THROW_EXCEPTION(std::logic_error("Surface does not support input"));
-    return server_input_channel->client_fd();
-}
-
-std::shared_ptr<mi::InputChannel> ms::BasicSurface::input_channel() const
-{
-    return server_input_channel;
-}
-
 void ms::BasicSurface::set_input_region(std::vector<geom::Rectangle> const& input_rectangles)
 {
     std::unique_lock<std::mutex> lock(guard);
@@ -432,7 +410,7 @@ void ms::BasicSurface::set_transformation(glm::mat4 const& t)
 bool ms::BasicSurface::visible() const
 {
     std::unique_lock<std::mutex> lk(guard);
-    return visible(lk); 
+    return visible(lk);
 }
 
 bool ms::BasicSurface::visible(std::unique_lock<std::mutex>&) const
@@ -458,7 +436,7 @@ void ms::BasicSurface::set_reception_mode(mi::InputReceptionMode mode)
 }
 
 MirWindowType ms::BasicSurface::type() const
-{    
+{
     std::unique_lock<std::mutex> lg(guard);
     return type_;
 }
@@ -466,7 +444,7 @@ MirWindowType ms::BasicSurface::type() const
 MirWindowType ms::BasicSurface::set_type(MirWindowType t)
 {
     std::unique_lock<std::mutex> lg(guard);
-    
+
     if (t < 0 || t > mir_window_types)
     {
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid surface "
@@ -478,7 +456,7 @@ MirWindowType ms::BasicSurface::set_type(MirWindowType t)
         type_ = t;
         lg.unlock();
 
-        observers.attrib_changed(mir_window_attrib_type, type_); 
+        observers.attrib_changed(mir_window_attrib_type, type_);
     }
 
     return t;
@@ -518,7 +496,7 @@ int ms::BasicSurface::set_swap_interval(int interval)
     {
         swapinterval_ = interval;
         bool allow_dropping = (interval == 0);
-        for (auto& info : layers) 
+        for (auto& info : layers)
             info.stream->allow_framedropping(allow_dropping);
 
         lg.unlock();
@@ -632,7 +610,7 @@ void ms::BasicSurface::set_cursor_image(std::shared_ptr<mg::CursorImage> const&
     {
         std::unique_lock<std::mutex> lock(guard);
         cursor_stream_adapter->reset();
-        
+
         cursor_image_ = image;
     }
 
@@ -744,7 +722,7 @@ int ms::BasicSurface::set_dpi(int new_dpi)
         lg.unlock();
         observers.attrib_changed(mir_window_attrib_dpi, new_dpi);
     }
-    
+
     return new_dpi;
 }
 
@@ -775,7 +753,7 @@ MirWindowVisibility ms::BasicSurface::set_visibility(MirWindowVisibility new_vis
 void ms::BasicSurface::add_observer(std::shared_ptr<SurfaceObserver> const& observer)
 {
     observers.add(observer);
-    for (auto& info : layers) 
+    for (auto& info : layers)
         info.stream->add_observer(observer);
 }
 
@@ -785,7 +763,7 @@ void ms::BasicSurface::remove_observer(std::weak_ptr<SurfaceObserver> const& obs
     if (!o)
         BOOST_THROW_EXCEPTION(std::runtime_error("Invalid observer (previously destroyed)"));
     observers.remove(o);
-    for (auto& info : layers) 
+    for (auto& info : layers)
         info.stream->remove_observer(observer);
 }
 
@@ -825,7 +803,7 @@ public:
     {
         return underlying_buffer_stream->framedropping() ? 0 : 1;
     }
- 
+
     std::shared_ptr<mg::Buffer> buffer() const override
     {
         if (!compositor_buffer)
@@ -844,7 +822,7 @@ public:
 
     bool shaped() const override
     { return mg::contains_alpha(underlying_buffer_stream->pixel_format()); }
- 
+
     mg::Renderable::ID id() const override
     { return id_; }
 private:
@@ -869,7 +847,7 @@ int ms::BasicSurface::buffers_ready_for_compositor(void const* id) const
 
 void ms::BasicSurface::consume(MirEvent const* event)
 {
-    input_validator.validate_and_dispatch(*event);
+    observers.input_consumed(event);
 }
 
 void ms::BasicSurface::set_keymap(MirInputDeviceId id, std::string const& model, std::string const& layout,
@@ -945,3 +923,8 @@ void ms::BasicSurface::placed_relative(geometry::Rectangle const& placement)
 {
     observers.placed_relative(placement);
 }
+
+void mir::scene::BasicSurface::start_drag_and_drop(std::vector<uint8_t> const& handle)
+{
+    observers.start_drag_and_drop(handle);
+}
