@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -104,3 +104,11 @@ void ms::LegacySurfaceChangeNotification::renamed(char const*)
 {
     notify_scene_change();
 }
+
+void ms::LegacySurfaceChangeNotification::input_consumed(MirEvent const*)
+{
+}
+
+void ms::LegacySurfaceChangeNotification::start_drag_and_drop(std::vector<uint8_t> const& /*handle*/)
+{
+}
\ No newline at end of file
