@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -148,12 +148,34 @@ int msh::FrontendShell::get_surface_attribute(
     return wrapped->get_surface_attribute(surface, attrib);
 }
 
-void msh::FrontendShell::raise_surface(
+void msh::FrontendShell::request_operation(
     std::shared_ptr<mf::Session> const& session,
     mf::SurfaceId surface_id,
-    uint64_t timestamp)
+    uint64_t timestamp,
+    UserRequest request)
 {
     auto const scene_session = std::dynamic_pointer_cast<ms::Session>(session);
     auto const surface = scene_session->surface(surface_id);
-    wrapped->raise_surface(scene_session, surface, timestamp);
+
+    switch (request)
+    {
+    case UserRequest::activate:
+    {
+        auto const scene_session = std::dynamic_pointer_cast<ms::Session>(session);
+        auto const surface = scene_session->surface(surface_id);
+        wrapped->raise_surface(scene_session, surface, timestamp);
+        break;
+    }
+
+    case UserRequest::drag_and_drop:
+        wrapped->request_drag_and_drop(scene_session, surface, timestamp);
+        break;
+
+    case UserRequest::move:
+        wrapped->request_move(scene_session, surface, timestamp);
+        break;
+
+    default:
+        break;
+    }
 }
