@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -87,10 +87,11 @@ struct FrontendShell : mf::Shell
         mf::SurfaceId surface_id,
         MirWindowAttrib attrib) override;
 
-    void raise_surface(
+    void request_operation(
         std::shared_ptr<mf::Session> const& session,
         mf::SurfaceId surface_id,
-        uint64_t timestamp) override;
+        uint64_t timestamp,
+        UserRequest request) override;
 };
 }
 }
