@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -120,6 +120,22 @@ void msh::ShellWrapper::raise_surface(
     wrapped->raise_surface(session, surface, timestamp);
 }
 
+void msh::ShellWrapper::request_drag_and_drop(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface,
+    uint64_t timestamp)
+{
+    wrapped->request_drag_and_drop(session, surface, timestamp);
+}
+
+void msh::ShellWrapper::request_move(
+    std::shared_ptr<ms::Session> const& session,
+    std::shared_ptr<ms::Surface> const& surface,
+    uint64_t timestamp)
+{
+    wrapped->request_move(session, surface, timestamp);
+}
+
 void msh::ShellWrapper::add_display(geometry::Rectangle const& area)
 {
     wrapped->add_display(area);
@@ -149,3 +165,13 @@ void msh::ShellWrapper::raise(SurfaceSet const& surfaces)
 {
     return wrapped->raise(surfaces);
 }
+
+void msh::ShellWrapper::set_drag_and_drop_handle(std::vector<uint8_t> const& handle)
+{
+    wrapped->set_drag_and_drop_handle(handle);
+}
+
+void msh::ShellWrapper::clear_drag_and_drop_handle()
+{
+    wrapped->clear_drag_and_drop_handle();
+}
