@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -201,3 +201,17 @@ void msh::SystemCompositorWindowManager::handle_raise_surface(
     uint64_t /*timestamp*/)
 {
 }
+
+void msh::SystemCompositorWindowManager::handle_request_drag_and_drop(
+    std::shared_ptr<ms::Session> const& /*session*/,
+    std::shared_ptr<ms::Surface> const& /*surface*/,
+    uint64_t /*timestamp*/)
+{
+}
+
+void msh::SystemCompositorWindowManager::handle_request_move(
+    std::shared_ptr<ms::Session> const& /*session*/,
+    std::shared_ptr<ms::Surface> const& /*surface*/,
+    uint64_t /*timestamp*/)
+{
+}
