@@ -2,7 +2,7 @@
  * Copyright © 2015-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,6 +21,11 @@
 #include "mir/input/device.h"
 #include "mir/input/event_filter.h"
 #include "mir/input/composite_event_filter.h"
+#include "mir/input/mir_input_config.h"
+#include "mir/shell/shell_wrapper.h"
+#include "mir/scene/session.h"
+#include "mir/scene/surface.h"
+#include "mir/graphics/event_handler_register.h"
 
 #include "mir_test_framework/fake_input_device.h"
 #include "mir_test_framework/stub_server_platform_factory.h"
@@ -29,6 +34,7 @@
 #include "mir_test_framework/any_surface.h"
 #include "mir/test/doubles/nested_mock_egl.h"
 #include "mir/test/doubles/mock_input_device_observer.h"
+#include "mir/test/doubles/fake_display.h"
 #include "mir/test/event_factory.h"
 #include "mir/test/event_matchers.h"
 #include "mir/test/fake_shared.h"
@@ -47,6 +53,7 @@ namespace mi = mir::input;
 namespace mis = mi::synthesis;
 
 namespace mt = mir::test;
+namespace mg = mir::graphics;
 namespace mtd = mt::doubles;
 namespace mtf = mir_test_framework;
 
@@ -55,14 +62,95 @@ using namespace std::chrono_literals;
 
 namespace
 {
+
+struct SurfaceTrackingShell : mir::shell::ShellWrapper
+{
+    SurfaceTrackingShell(
+        std::shared_ptr<mir::shell::Shell> wrapped_shell)
+        : ShellWrapper{wrapped_shell}, wrapped_shell{wrapped_shell}
+    {}
+
+    mir::frontend::SurfaceId create_surface(
+        std::shared_ptr<mir::scene::Session> const& session,
+        mir::scene::SurfaceCreationParameters const& params,
+        std::shared_ptr<mir::frontend::EventSink> const& sink) override
+    {
+        auto surface_id = wrapped_shell->create_surface(session, params, sink);
+
+        tracked_surfaces[session->name()] =  TrackedSurface{session, surface_id};
+
+        return surface_id;
+    }
+
+    std::shared_ptr<mir::scene::Surface> get_surface(std::string const& session_name)
+    {
+        if (end(tracked_surfaces) == tracked_surfaces.find(session_name))
+            return nullptr;
+        TrackedSurface & tracked_surface = tracked_surfaces[session_name];
+        auto session = tracked_surface.session.lock();
+        if (!session)
+            return nullptr;
+        return session->surface(tracked_surface.surface);
+    }
+
+    struct TrackedSurface
+    {
+        std::weak_ptr<mir::scene::Session> session;
+        mir::frontend::SurfaceId surface;
+    };
+    std::unordered_map<std::string, TrackedSurface> tracked_surfaces;
+    std::shared_ptr<mir::shell::Shell> wrapped_shell;
+};
+
+struct Display : mtd::FakeDisplay
+{
+    using mtd::FakeDisplay::FakeDisplay;
+
+    void register_pause_resume_handlers(
+                     mg::EventHandlerRegister&,
+                     mg::DisplayPauseHandler const& pause_handler,
+                     mg::DisplayResumeHandler const& resume_handler) override
+    {
+        pause_function = pause_handler;
+        resume_function = resume_handler;
+    }
+
+    void trigger_pause()
+    {
+        pause_function();
+    }
+    void trigger_resume()
+    {
+        resume_function();
+    }
+    std::function<bool()> pause_function{[](){return false;}};
+    std::function<bool()> resume_function{[](){return false;}};
+};
+
+size_t get_index_of(MirInputConfig const* config, std::string const& id)
+{
+    for (size_t i = 0, e = mir_input_config_device_count(config);i != e;++i)
+    {
+        auto const* device = mir_input_config_get_device(config, i);
+        if (mir_input_device_get_unique_id(device) == id)
+            return i;
+    }
+    return mir_input_config_device_count(config);
+}
+
+bool contains(MirInputConfig const* config, std::string const& id)
+{
+    return mir_input_config_device_count(config) != get_index_of(config, id);
+}
+
 struct MockEventFilter : public mi::EventFilter
 {
     // Work around GMock wanting to know how to construct MirEvent
-    MOCK_METHOD1(handle, bool(MirEvent const*));
+    MOCK_METHOD1(handle, void(MirEvent const*));
     bool handle(MirEvent const& ev)
     {
         handle(&ev);
-        return true;
+        return false;
     }
 };
 
@@ -77,7 +165,15 @@ struct NestedServerWithMockEventFilter : mtf::HeadlessNestedServerRunner
                                     std::shared_ptr<MockEventFilter> const& event_filter)
         : mtf::HeadlessNestedServerRunner(connection_string), mock_event_filter{event_filter}
     {
-        server.the_composite_event_filter()->append(mock_event_filter);
+        server.the_composite_event_filter()->prepend(mock_event_filter);
+        server.wrap_shell(
+            [this](std::shared_ptr<mir::shell::Shell> const& wrapped)
+            {
+                surfaces = std::make_shared<SurfaceTrackingShell>(wrapped);
+                return surfaces;
+            });
+
+
         start_server();
     }
     NestedServerWithMockEventFilter(std::string const& connection_string)
@@ -90,18 +186,27 @@ struct NestedServerWithMockEventFilter : mtf::HeadlessNestedServerRunner
         stop_server();
     }
 
+    std::shared_ptr<mir::scene::Surface> get_surface(std::string const& name)
+    {
+        return surfaces->get_surface(name);
+    }
+    std::shared_ptr<SurfaceTrackingShell> surfaces;
     std::shared_ptr<MockEventFilter> const mock_event_filter;
 };
 
 struct NestedInput : public mtf::HeadlessInProcessServer
 {
+    Display display{display_geometry};
+    NestedInput()
+    {
+        preset_display(mt::fake_shared(display));
+    }
 
     void SetUp()
     {
-        initial_display_layout(display_geometry);
         mtf::HeadlessInProcessServer::SetUp();
     }
-    
+
     mtd::NestedMockEGL mock_egl;
 
     std::unique_ptr<mtf::FakeInputDevice> fake_keyboard{
@@ -126,16 +231,32 @@ struct NestedInputWithMouse : NestedInput
 struct ExposedSurface
 {
 public:
-    ExposedSurface(std::string const& connect_string)
+    using UniqueInputConfig = std::unique_ptr<MirInputConfig , void(*)(MirInputConfig const*)>;
+    ExposedSurface(std::string const& connect_string, std::string const& session_name = "ExposedSurface")
     {
         // Ensure the nested server posts a frame
-        connection = mir_connect_sync(connect_string.c_str(), __PRETTY_FUNCTION__);
-        window = mtf::make_any_surface(connection);
-        mir_window_set_event_handler(window, handle_event, this);
+        connection = mir_connect_sync(connect_string.c_str(), session_name.c_str());
+
+        mir_connection_set_input_config_change_callback(
+            connection,
+            [](MirConnection*, void* context)
+            {
+                auto* surf = static_cast<ExposedSurface*>(context);
+                auto config = surf->get_input_config();
+                surf->input_config(config.get());
+            },
+            this);
+
+        window = mtf::make_any_surface(connection, handle_event, this);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
         mir_buffer_stream_swap_buffers_sync(mir_window_get_buffer_stream(window));
+#pragma GCC diagnostic pop
     }
 
     MOCK_METHOD1(handle_input, void(MirEvent const*));
+    MOCK_METHOD1(handle_device_state, void(MirEvent const*));
+    MOCK_METHOD0(handle_keymap, void());
 
     void handle_window_event(MirWindowEvent const* event)
     {
@@ -150,10 +271,31 @@ public:
             mir_window_focus_state_focused == value)
             focused = true;
 
-        if (exposed && focused)
+        test_and_raise();
+    }
+
+    void test_and_raise()
+    {
+        if (exposed && focused && input_device_state_received)
             ready_to_accept_events.raise();
     }
 
+    UniqueInputConfig get_input_config()
+    {
+        auto config = mir_connection_create_input_config(connection);
+        return {config,&mir_input_config_release};
+    }
+
+    void on_input_config(std::function<void(MirInputConfig const*)> const& handler)
+    {
+        input_config = handler;
+    }
+
+    void apply_config(MirInputConfig* configuration)
+    {
+        mir_connection_apply_session_input_config(connection, configuration);
+    }
+
     static void handle_event(MirWindow*, MirEvent const* ev, void* context)
     {
         auto const client = static_cast<ExposedSurface*>(context);
@@ -164,8 +306,17 @@ public:
             client->handle_window_event(window_event);
 
         }
+        if (type == mir_event_type_keymap)
+            client->handle_keymap();
         if (type == mir_event_type_input)
             client->handle_input(ev);
+        if (type == mir_event_type_input_device_state)
+        {
+            client->input_device_state_received = true;
+            client->test_and_raise();
+            client->handle_device_state(ev);
+        }
+
     }
 
     static void null_event_handler(MirWindow*, MirEvent const*, void*) {};
@@ -180,12 +331,14 @@ public:
 protected:
     ExposedSurface(ExposedSurface const&) = delete;
     ExposedSurface& operator=(ExposedSurface const&) = delete;
-    
+
 private:
     MirConnection *connection;
     MirWindow *window;
     bool exposed{false};
     bool focused{false};
+    bool input_device_state_received = false;
+    std::function<void(MirInputConfig const* confg)> input_config{[](MirInputConfig const*){}};
 };
 
 }
@@ -195,21 +348,24 @@ TEST_F(NestedInput, nested_event_filter_receives_keyboard_from_host)
     MockEventFilter nested_event_filter;
     std::atomic<int> num_key_a_events{0};
     auto const increase_key_a_events = [&num_key_a_events] { ++num_key_a_events; };
+    mt::Signal devices_ready;
 
     InSequence seq;
-    EXPECT_CALL(nested_event_filter, handle(mt::InputDeviceStateEvent()));
+    EXPECT_CALL(nested_event_filter, handle(mt::InputDeviceStateEvent()))
+        .WillOnce(mt::WakeUp(&devices_ready));
     EXPECT_CALL(nested_event_filter, handle(mt::KeyOfScanCode(KEY_A))).
         Times(AtLeast(1)).
-        WillRepeatedly(DoAll(InvokeWithoutArgs(increase_key_a_events), Return(true)));
+        WillRepeatedly(InvokeWithoutArgs(increase_key_a_events));
 
     EXPECT_CALL(nested_event_filter, handle(mt::KeyOfScanCode(KEY_RIGHTSHIFT))).
         Times(2).
-        WillOnce(Return(true)).
-        WillOnce(DoAll(mt::WakeUp(&all_events_received), Return(true)));
+        WillOnce(mt::WakeUp(&all_events_received));
 
     NestedServerWithMockEventFilter nested_mir{new_connection(), mt::fake_shared(nested_event_filter)};
     ExposedSurface client(nested_mir.new_connection());
 
+    ASSERT_TRUE(devices_ready.wait_for(60s));
+
     // Because we are testing a nested setup, it's difficult to guarantee
     // that the nested framebuffer window (and consenquently the client window
     // contained in it) is going to be ready (i.e., exposed and focused) to receive
@@ -223,7 +379,7 @@ TEST_F(NestedInput, nested_event_filter_receives_keyboard_from_host)
             fake_keyboard->emit_event(mis::a_key_up_event().of_scancode(KEY_A));
             return false;
         },
-        std::chrono::seconds{5});
+        std::chrono::seconds{10});
 
     EXPECT_TRUE(dummy_events_received);
 
@@ -252,13 +408,12 @@ TEST_F(NestedInput, on_add_device_observer_gets_device_added_calls_on_existing_d
     NestedServerWithMockEventFilter nested_mir{new_connection()};
     auto nested_hub = nested_mir.server.the_input_device_hub();
 
-    EXPECT_CALL(*mock_observer, device_added(_)).Times(1);
-    EXPECT_CALL(*mock_observer, changes_complete())
-        .Times(1)
+    EXPECT_CALL(*mock_observer, device_added(_))
         .WillOnce(mt::WakeUp(&input_device_changes_complete));
 
     nested_hub->add_observer(mock_observer);
     input_device_changes_complete.wait_for(10s);
+    nested_hub->remove_observer(mock_observer);
 }
 
 TEST_F(NestedInput, device_added_on_host_triggeres_nested_device_observer)
@@ -266,15 +421,18 @@ TEST_F(NestedInput, device_added_on_host_triggeres_nested_device_observer)
     NestedServerWithMockEventFilter nested_mir{new_connection()};
     auto nested_hub = nested_mir.server.the_input_device_hub();
 
-    EXPECT_CALL(*mock_observer, changes_complete()).Times(1)
+    // wait until we see the keyboard from the fixture:
+    EXPECT_CALL(*mock_observer, device_added(_)).Times(1)
         .WillOnce(mt::WakeUp(&input_device_changes_complete));
     nested_hub->add_observer(mock_observer);
 
     input_device_changes_complete.wait_for(10s);
     EXPECT_THAT(input_device_changes_complete.raised(), Eq(true));
     input_device_changes_complete.reset();
+    ::testing::Mock::VerifyAndClearExpectations(&mock_observer);
 
-    EXPECT_CALL(*mock_observer, changes_complete())
+    // wait for the fake mouse
+    EXPECT_CALL(*mock_observer, device_added(_)).Times(1)
         .WillOnce(mt::WakeUp(&input_device_changes_complete));
 
     auto mouse = mtf::add_fake_input_device(mi::InputDeviceInfo{"mouse", "mouse-uid" , mi::DeviceCapability::pointer});
@@ -298,7 +456,7 @@ TEST_F(NestedInput, on_input_device_state_nested_server_emits_input_device_state
         .WillOnce(mt::WakeUp(&client_to_host_event_received));
     EXPECT_CALL(nested_event_filter,
                 handle(mt::DeviceStateWithPressedKeys(std::vector<uint32_t>({KEY_LEFTALT, KEY_TAB}))))
-        .WillOnce(DoAll(mt::WakeUp(&client_to_nested_event_received), Return(true)));
+        .WillOnce(mt::WakeUp(&client_to_nested_event_received));
 
     fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_LEFTALT));
     fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_TAB));
@@ -317,12 +475,12 @@ TEST_F(NestedInputWithMouse, mouse_pointer_coordinates_in_nested_server_are_accu
     mt::Signal devices_ready;
     mt::Signal event_received;
     EXPECT_CALL(nested_event_filter, handle(mt::InputDeviceStateEvent()))
-        .WillOnce(DoAll(mt::WakeUp(&devices_ready), Return(true)));
+        .WillOnce(mt::WakeUp(&devices_ready));
 
     EXPECT_CALL(nested_event_filter,
                 handle(AllOf(mt::PointerEventWithPosition(initial_movement_x, initial_movement_y),
                              mt::PointerEventWithDiff(initial_movement_x, initial_movement_y))))
-        .WillOnce(DoAll(mt::WakeUp(&event_received), Return(true)));
+        .WillOnce(mt::WakeUp(&event_received));
 
     NestedServerWithMockEventFilter nested_mir{new_connection(), mt::fake_shared(nested_event_filter)};
     ExposedSurface client_to_nested_mir(nested_mir.new_connection());
@@ -338,7 +496,7 @@ TEST_F(NestedInputWithMouse, mouse_pointer_coordinates_in_nested_server_are_accu
     EXPECT_CALL(nested_event_filter,
                 handle(AllOf(mt::PointerEventWithPosition(initial_movement_x + second_movement_x, initial_movement_y + second_movement_y),
                              mt::PointerEventWithDiff(second_movement_x, second_movement_y))))
-        .WillOnce(DoAll(mt::WakeUp(&event_received), Return(true)));
+        .WillOnce(mt::WakeUp(&event_received));
 
     fake_mouse->emit_event(mis::a_pointer_event().with_movement(second_movement_x, second_movement_y));
     ASSERT_TRUE(event_received.wait_for(60s));
@@ -360,11 +518,11 @@ TEST_F(NestedInputWithMouse, mouse_pointer_position_is_in_sync_with_host_server)
 
     EXPECT_CALL(nested_event_filter, handle(
             mt::DeviceStateWithPosition(initial_x, initial_y)))
-        .WillOnce(DoAll(mt::WakeUp(&devices_ready), Return(true)));
+        .WillOnce(mt::WakeUp(&devices_ready));
     EXPECT_CALL(nested_event_filter,
                 handle(AllOf(mt::PointerEventWithPosition(final_x, final_y),
                              mt::PointerEventWithDiff(x[2], y[2]))))
-        .WillOnce(DoAll(mt::WakeUp(&event_received), Return(true)));
+        .WillOnce(mt::WakeUp(&event_received));
 
     NestedServerWithMockEventFilter nested_mir{new_connection(), mt::fake_shared(nested_event_filter)};
     ExposedSurface client_to_nested_mir(nested_mir.new_connection());
@@ -374,3 +532,84 @@ TEST_F(NestedInputWithMouse, mouse_pointer_position_is_in_sync_with_host_server)
     fake_mouse->emit_event(mis::a_pointer_event().with_movement(x[2], y[2]));
     ASSERT_TRUE(event_received.wait_for(60s));
 }
+
+TEST_F(NestedInput, nested_clients_can_change_host_device_configurations)
+{
+    auto const acceleration_bias = 0.9f;
+    std::string const uid{"mouse-uid"};
+
+    mt::Signal fake_device_received;
+    NestedServerWithMockEventFilter nested_mir{new_connection()};
+    ExposedSurface client_to_nested(nested_mir.new_connection());
+    client_to_nested.on_input_config(
+        [&](MirInputConfig const* config)
+        {
+            if (contains(config, uid))
+                fake_device_received.raise();
+        });
+    client_to_nested.ready_to_accept_events.wait_for(4s);
+
+    std::unique_ptr<mtf::FakeInputDevice> fake_mouse{
+        mtf::add_fake_input_device(mi::InputDeviceInfo{"mouse", uid, mi::DeviceCapability::pointer})
+    };
+
+    ASSERT_TRUE(fake_device_received.wait_for(4s));
+
+    auto device_config = client_to_nested.get_input_config();
+    auto device = mir_input_config_get_mutable_device(device_config.get(), get_index_of(device_config.get(), uid));
+    auto ptr_conf = mir_input_device_get_mutable_pointer_config(device);
+
+    mir_pointer_config_set_acceleration_bias(ptr_conf, acceleration_bias);
+    fake_device_received.reset();
+    client_to_nested.apply_config(device_config.get());
+
+    EXPECT_TRUE(fake_device_received.wait_for(2s));
+
+    device_config = client_to_nested.get_input_config();
+    device = mir_input_config_get_mutable_device(device_config.get(), get_index_of(device_config.get(), uid));
+    ptr_conf = mir_input_device_get_mutable_pointer_config(device);
+
+    EXPECT_THAT(mir_pointer_config_get_acceleration_bias(ptr_conf), acceleration_bias);
+}
+
+TEST_F(NestedInput, pressed_keys_on_vt_switch_are_forgotten)
+{
+    mt::Signal keymap_received;
+    mt::Signal devices_ready;
+    mt::Signal initial_keys_received;
+    mt::Signal keys_without_modifier_received;
+    NiceMock<MockEventFilter> nested_event_filter;
+    ON_CALL(nested_event_filter, handle(
+            mt::InputDeviceStateEvent()))
+        .WillByDefault(mt::WakeUp(&devices_ready));
+
+    NestedServerWithMockEventFilter nested_mir{new_connection(), mt::fake_shared(nested_event_filter)};
+    ExposedSurface client_to_nested(nested_mir.new_connection(), "with_keymap");
+
+    ASSERT_TRUE(devices_ready.wait_for(10s));
+    EXPECT_CALL(client_to_nested, handle_keymap())
+        .WillOnce(mt::WakeUp(&keymap_received));
+    EXPECT_CALL(client_to_nested, handle_input(mt::KeyOfScanCode(KEY_RIGHTALT)));
+    EXPECT_CALL(client_to_nested, handle_input(mt::KeyOfScanCode(KEY_RIGHTCTRL)))
+        .WillOnce(mt::WakeUp(&initial_keys_received));
+
+    nested_mir.get_surface("with_keymap")->set_keymap(MirInputDeviceId{0}, "pc105", "de", "", "");
+
+    ASSERT_TRUE(client_to_nested.ready_to_accept_events.wait_for(10s));
+    EXPECT_TRUE(keymap_received.wait_for(10s));
+
+    fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_RIGHTALT));
+    fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_RIGHTCTRL));
+
+    EXPECT_TRUE(initial_keys_received.wait_for(10s));
+
+    display.trigger_pause();
+    display.trigger_resume();
+
+    EXPECT_CALL(client_to_nested,
+                handle_input(AllOf(mt::KeyOfScanCode(KEY_A), mt::KeyWithModifiers(mir_input_event_modifier_none))))
+        .WillOnce(mt::WakeUp(&keys_without_modifier_received));
+
+    fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_A));
+    EXPECT_TRUE(keys_without_modifier_received.wait_for(10s));
+}
