@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,7 +21,7 @@
 #include "mir/scene/null_surface_observer.h"
 
 #include "mir/test/doubles/wrap_shell_to_track_latest_surface.h"
-#include "mir_test_framework/connected_client_with_a_surface.h"
+#include "mir_test_framework/connected_client_with_a_window.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/signal.h"
 
@@ -51,7 +51,7 @@ public:
     MOCK_METHOD1(hidden_set_to, void(bool));
 };
 
-struct SurfaceModifications : mtf::ConnectedClientWithASurface
+struct SurfaceModifications : mtf::ConnectedClientWithAWindow
 {
     SurfaceModifications() { add_to_environment("MIR_SERVER_ENABLE_INPUT", "OFF"); }
 
@@ -66,14 +66,17 @@ struct SurfaceModifications : mtf::ConnectedClientWithASurface
             return msc;
         });
 
-        mtf::ConnectedClientWithASurface::SetUp();
+        mtf::ConnectedClientWithAWindow::SetUp();
 
         shell_surface = shell->latest_surface;
         auto const scene_surface = shell_surface.lock();
         scene_surface->add_observer(mt::fake_shared(surface_observer));
 
         // Swap buffers to ensure window is visible for event based tests
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
         mir_buffer_stream_swap_buffers_sync(mir_window_get_buffer_stream(window));
+#pragma GCC diagnostic pop
     }
 
     void generate_alt_click_at(Point const& click_position)
