@@ -2,7 +2,7 @@
  * Copyright © 2013-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -72,6 +72,9 @@ struct MockShell : public frontend::Shell
 
     MOCK_METHOD3(raise_surface, void(std::shared_ptr<frontend::Session> const& session,
         frontend::SurfaceId surface_id, uint64_t timestamp));
+
+    MOCK_METHOD4(request_operation, void(std::shared_ptr<frontend::Session> const &session,
+        frontend::SurfaceId surface_id, uint64_t timestamp, UserRequest request));
 };
 
 }
