@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 
 #include "stub_buffer.h"
 #include "mir/renderer/gl/texture_source.h"
+#include "mir/renderer/gl/texture_target.h"
 
 namespace mir
 {
@@ -30,7 +31,8 @@ namespace doubles
 {
 
 class StubGLBuffer : public StubBuffer,
-                     public renderer::gl::TextureSource
+                     public renderer::gl::TextureSource,
+                     public renderer::gl::TextureTarget
 {
 public:
     using StubBuffer::StubBuffer;
@@ -38,6 +40,7 @@ public:
     void gl_bind_to_texture() {}
     void bind() {}
     void secure_for_render() {}
+    void bind_for_write() {}
 };
 
 }
