@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -181,6 +181,9 @@ public:
     {
         return true;
     }
+    void apply_input_configuration(MirInputConfig const*)
+    {
+    }
     optional_value<std::shared_ptr<graphics::MesaAuthExtension>> auth_extension()
     {
         return {};
@@ -189,7 +192,12 @@ public:
     {
         return {};
     }
+    optional_value<mir::Fd> drm_fd()
+    {
+        return {};
+    }
     void* request_interface(char const*, int) { return nullptr; }
+    std::vector<mir::ExtensionDescription> extensions() const { return {}; }
 };
 
 struct MockHostConnection : StubHostConnection
@@ -202,12 +210,14 @@ struct MockHostConnection : StubHostConnection
         (std::shared_ptr<graphics::nested::HostStream> const&, geometry::Displacement,
          graphics::BufferProperties, char const*, uint32_t));
     MOCK_METHOD2(request_interface, void*(char const*, int));
+    MOCK_METHOD1(apply_input_configuration, void(MirInputConfig const*));
 
     void emit_input_event(MirEvent const& event, mir::geometry::Rectangle const& source_frame)
     {
         if (event_callback)
             event_callback(event, source_frame);
     }
+    std::vector<ExtensionDescription> extensions() const { return {}; }
 
     MockHostConnection()
     {
