@@ -2,7 +2,7 @@
  * Copyright © 2013 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -36,6 +36,9 @@ struct StubInputTargeter : public shell::InputTargeter
     void clear_focus()
     {
     }
+
+    void set_drag_and_drop_handle(std::vector<uint8_t> const&) override {}
+    void clear_drag_and_drop_handle() override {}
 };
 
 }
