@@ -2,7 +2,7 @@
  * Copyright © 2015-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,24 +18,29 @@
 
 #include "src/server/input/default_input_device_hub.h"
 #include "src/server/input/basic_seat.h"
+#include "src/server/input/config_changer.h"
+#include "src/server/scene/broadcasting_session_event_sink.h"
 
 #include "mir/test/doubles/mock_input_device.h"
 #include "mir/test/doubles/mock_input_device_observer.h"
 #include "mir/test/doubles/mock_input_dispatcher.h"
-#include "mir/test/doubles/mock_input_region.h"
 #include "mir/test/doubles/mock_touch_visualizer.h"
 #include "mir/test/doubles/mock_cursor_listener.h"
-#include "mir/test/doubles/mock_event_sink.h"
+#include "mir/test/doubles/mock_input_manager.h"
 #include "mir/test/doubles/mock_seat_report.h"
 #include "mir/test/doubles/mock_server_status_listener.h"
+#include "mir/test/doubles/mock_scene_session.h"
+#include "mir/test/doubles/stub_session_container.h"
 #include "mir/test/doubles/triggered_main_loop.h"
 #include "mir/test/event_matchers.h"
 #include "mir/test/doubles/advanceable_clock.h"
 #include "mir/test/fake_shared.h"
+#include "mir/test/doubles/stub_display_configuration.h"
 
 #include "mir/dispatch/multiplexing_dispatchable.h"
 #include "mir/cookie/authority.h"
 #include "mir/graphics/buffer.h"
+#include "mir/graphics/display_configuration_observer.h"
 
 #include "mir/input/device.h"
 #include "mir/input/xkb_mapper.h"
@@ -46,6 +51,7 @@
 #include "mir/input/input_device_info.h"
 #include "mir/geometry/rectangles.h"
 #include "mir/test/input_config_matchers.h"
+#include "mir/test/fd_utils.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -53,6 +59,7 @@
 namespace mt = mir::test;
 namespace mtd = mt::doubles;
 namespace mi = mir::input;
+namespace ms = mir::scene;
 namespace geom = mir::geometry;
 using namespace std::literals::chrono_literals;
 using namespace ::testing;
@@ -68,18 +75,47 @@ void PrintTo(mir::input::InputDeviceInfo const &info, ::std::ostream* out)
 }
 }
 
+namespace
+{
+
 MATCHER_P(DeviceMatches, device_info, "")
 {
     return arg.name() == device_info.name &&
         arg.unique_id() == device_info.unique_id;
 }
 
+struct FakeDisplayConfigurationObserverRegistrar : mir::ObserverRegistrar<mir::graphics::DisplayConfigurationObserver>
+{
+    using Observer = mir::graphics::DisplayConfigurationObserver;
+    mtd::StubDisplayConfig output{{geom::Rectangle{{0, 0}, {100, 100}}}};
+    void register_interest(std::weak_ptr<Observer> const& obs) override
+    {
+        observer = obs;
+        auto o = observer.lock();
+        o->initial_configuration(mt::fake_shared(output));
+    }
+    void register_interest(
+        std::weak_ptr<Observer> const& obs,
+        mir::Executor&) override
+    {
+        observer = obs;
+    }
+    void unregister_interest(Observer const&) override
+    {
+        observer.reset();
+    }
+    void update_output(geom::Size const& output_size)
+    {
+        output.outputs[0].modes[0].size = output_size;
+        auto o = observer.lock();
+        o->configuration_applied(mt::fake_shared(output));
+    }
+    std::weak_ptr<Observer> observer;
+};
 struct SingleSeatInputDeviceHubSetup : ::testing::Test
 {
     mtd::TriggeredMainLoop observer_loop;
     NiceMock<mtd::MockInputDispatcher> mock_dispatcher;
-    NiceMock<mtd::MockInputRegion> mock_region;
-    NiceMock<mtd::MockEventSink> mock_sink;
     std::shared_ptr<mir::cookie::Authority> cookie_authority = mir::cookie::Authority::create();
     NiceMock<mtd::MockCursorListener> mock_cursor_listener;
     NiceMock<mtd::MockTouchVisualizer> mock_visualizer;
@@ -88,13 +124,25 @@ struct SingleSeatInputDeviceHubSetup : ::testing::Test
     mi::receiver::XKBMapper key_mapper;
     mir::dispatch::MultiplexingDispatchable multiplexer;
     mtd::AdvanceableClock clock;
-    mi::BasicSeat seat{
-        mt::fake_shared(mock_dispatcher),mt::fake_shared(mock_visualizer), mt::fake_shared(mock_cursor_listener),
-        mt::fake_shared(mock_region), mt::fake_shared(key_mapper), mt::fake_shared(clock), mt::fake_shared(mock_seat_observer)};
-    mi::DefaultInputDeviceHub hub{
-        mt::fake_shared(mock_sink), mt::fake_shared(seat), mt::fake_shared(multiplexer), mt::fake_shared(observer_loop),
-        cookie_authority, mt::fake_shared(key_mapper), mt::fake_shared(mock_status_listener)};
+    mtd::MockInputManager mock_input_manager;
+    mtd::StubSessionContainer stub_session_container;
+    ms::BroadcastingSessionEventSink session_event_sink;
+    FakeDisplayConfigurationObserverRegistrar display_config;
+    mi::BasicSeat seat{mt::fake_shared(mock_dispatcher),      mt::fake_shared(mock_visualizer),
+                       mt::fake_shared(mock_cursor_listener), mt::fake_shared(display_config),
+                       mt::fake_shared(key_mapper),           mt::fake_shared(clock),
+                       mt::fake_shared(mock_seat_observer)};
+    mi::DefaultInputDeviceHub hub{mt::fake_shared(seat), mt::fake_shared(multiplexer),
+                                  cookie_authority,      mt::fake_shared(key_mapper),
+                                  mt::fake_shared(mock_status_listener)};
     NiceMock<mtd::MockInputDeviceObserver> mock_observer;
+    mi::ConfigChanger changer{
+        mt::fake_shared(mock_input_manager),
+            mt::fake_shared(hub),
+            mt::fake_shared(stub_session_container),
+            mt::fake_shared(session_event_sink),
+            mt::fake_shared(hub)
+    };
 
     mi::DeviceCapabilities const keyboard_caps = mi::DeviceCapability::keyboard | mi::DeviceCapability::alpha_numeric;
     mi::DeviceCapabilities const touchpad_caps = mi::DeviceCapability::touchpad | mi::DeviceCapability::pointer;
@@ -105,6 +153,21 @@ struct SingleSeatInputDeviceHubSetup : ::testing::Test
 
     std::chrono::nanoseconds arbitrary_timestamp;
 
+    void SetUp()
+    {
+        // execute registration of ConfigChanger
+        expect_and_execute_multiplexer();
+    }
+
+    void expect_and_execute_multiplexer(int count = 1)
+    {
+        for (int i = 0; i != count; ++i)
+        {
+            mt::fd_becomes_readable(multiplexer.watch_fd(), 5s);
+            multiplexer.dispatch(mir::dispatch::FdEvent::readable);
+        }
+    }
+
     void capture_input_sink(NiceMock<mtd::MockInputDevice>& dev, mi::InputSink*& sink, mi::EventBuilder*& builder)
     {
         ON_CALL(dev,start(_,_))
@@ -117,6 +180,8 @@ struct SingleSeatInputDeviceHubSetup : ::testing::Test
     }
 };
 
+}
+
 TEST_F(SingleSeatInputDeviceHubSetup, input_sink_posts_events_to_input_dispatcher)
 {
     mi::InputSink* sink;
@@ -125,14 +190,13 @@ TEST_F(SingleSeatInputDeviceHubSetup, input_sink_posts_events_to_input_dispatche
 
     capture_input_sink(device, sink, builder);
 
-    EXPECT_CALL(mock_observer,device_added(_))
+    EXPECT_CALL(mock_observer, device_added(_))
         .WillOnce(SaveArg<0>(&handle));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer(1);
     hub.add_device(mt::fake_shared(device));
 
-    observer_loop.trigger_server_actions();
-
     auto event = builder->key_event(arbitrary_timestamp, mir_keyboard_action_down, 0,
                                     KEY_A);
 
@@ -150,8 +214,6 @@ TEST_F(SingleSeatInputDeviceHubSetup, forwards_touch_spots_to_visualizer)
 
     hub.add_device(mt::fake_shared(device));
 
-    observer_loop.trigger_server_actions();
-
     auto touch_event_1 = builder->touch_event(
         arbitrary_timestamp,
         {{0, mir_touch_action_down, mir_touch_tooltype_finger, 21.0f, 34.0f, 50.0f, 15.0f, 5.0f, 4.0f}});
@@ -185,20 +247,18 @@ TEST_F(SingleSeatInputDeviceHubSetup, forwards_touch_spots_to_visualizer)
     sink->handle_input(*touch_event_4);
 }
 
-
 TEST_F(SingleSeatInputDeviceHubSetup, tracks_pointer_position)
 {
     geom::Point first{10,10}, second{20,20}, third{10,30};
-    EXPECT_CALL(mock_region, confine(first));
-    EXPECT_CALL(mock_region, confine(second));
-    EXPECT_CALL(mock_region, confine(third));
+    EXPECT_CALL(mock_cursor_listener, cursor_moved_to(first.x.as_int(), first.y.as_int()));
+    EXPECT_CALL(mock_cursor_listener, cursor_moved_to(second.x.as_int(), second.y.as_int()));
+    EXPECT_CALL(mock_cursor_listener, cursor_moved_to(third.x.as_int(), third.y.as_int()));
 
     mi::InputSink* sink;
     mi::EventBuilder* builder;
     capture_input_sink(device, sink, builder);
 
     hub.add_device(mt::fake_shared(device));
-    observer_loop.trigger_server_actions();
     sink->handle_input(
         *builder->pointer_event(arbitrary_timestamp, mir_pointer_action_motion, 0, 0.0f, 0.0f, 10.0f, 10.0f));
     sink->handle_input(
@@ -210,16 +270,14 @@ TEST_F(SingleSeatInputDeviceHubSetup, tracks_pointer_position)
 TEST_F(SingleSeatInputDeviceHubSetup, confines_pointer_movement)
 {
     geom::Point confined_pos{10, 18};
+    display_config.update_output(geom::Size{confined_pos.x.as_int() + 1, confined_pos.y.as_int() + 1});
 
-    ON_CALL(mock_region,confine(_))
-        .WillByDefault(SetArgReferee<0>(confined_pos));
     EXPECT_CALL(mock_cursor_listener, cursor_moved_to(confined_pos.x.as_int(), confined_pos.y.as_int())).Times(2);
 
     mi::InputSink* sink;
     mi::EventBuilder* builder;
     capture_input_sink(device, sink, builder);
     hub.add_device(mt::fake_shared(device));
-    observer_loop.trigger_server_actions();
 
     sink->handle_input(
         *builder->pointer_event(arbitrary_timestamp, mir_pointer_action_motion, 0, 0.0f, 0.0f, 10.0f, 20.0f));
@@ -249,8 +307,8 @@ TEST_F(SingleSeatInputDeviceHubSetup, forwards_pointer_settings_to_input_device)
     ON_CALL(mock_observer, device_added(_)).WillByDefault(SaveArg<0>(&dev));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer();
     hub.add_device(mt::fake_shared(touchpad));
-    observer_loop.trigger_server_actions();
 
     EXPECT_CALL(touchpad, apply_settings(Matcher<mi::PointerSettings const&>(_)));
 
@@ -265,8 +323,8 @@ TEST_F(SingleSeatInputDeviceHubSetup, forwards_touchpad_settings_to_input_device
     ON_CALL(mock_observer, device_added(_)).WillByDefault(SaveArg<0>(&dev));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer();
     hub.add_device(mt::fake_shared(touchpad));
-    observer_loop.trigger_server_actions();
 
     EXPECT_CALL(touchpad, apply_settings(Matcher<mi::TouchpadSettings const&>(_)));
 
@@ -288,9 +346,9 @@ TEST_F(SingleSeatInputDeviceHubSetup, input_sink_tracks_modifier)
         .WillOnce(SaveArg<0>(&key_handle));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer();
     hub.add_device(mt::fake_shared(another_device));
 
-    observer_loop.trigger_server_actions();
 
     const MirInputEventModifiers shift_left = mir_input_event_modifier_shift_left | mir_input_event_modifier_shift;
     auto shift_down =
@@ -324,11 +382,10 @@ TEST_F(SingleSeatInputDeviceHubSetup, input_sink_unifies_modifier_state_accross_
         .WillOnce(SaveArg<0>(&key_handle));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer();
     hub.add_device(mt::fake_shared(device));
     hub.add_device(mt::fake_shared(another_device));
 
-    observer_loop.trigger_server_actions();
-
     const MirInputEventModifiers r_alt_modifier = mir_input_event_modifier_alt_right | mir_input_event_modifier_alt;
     auto key =
         key_event_builder->key_event(arbitrary_timestamp, mir_keyboard_action_down, 0, KEY_RIGHTALT);
@@ -369,6 +426,7 @@ TEST_F(SingleSeatInputDeviceHubSetup, input_sink_reduces_modifier_state_accross_
         .WillOnce(SaveArg<0>(&key_handle_2));
 
     hub.add_observer(mt::fake_shared(mock_observer));
+    expect_and_execute_multiplexer();
     hub.add_device(mt::fake_shared(device));
     hub.add_device(mt::fake_shared(another_device));
     hub.add_device(mt::fake_shared(third_device));
@@ -377,8 +435,6 @@ TEST_F(SingleSeatInputDeviceHubSetup, input_sink_reduces_modifier_state_accross_
     const MirInputEventModifiers l_ctrl_modifier = mir_input_event_modifier_ctrl_left | mir_input_event_modifier_ctrl;
     const MirInputEventModifiers combined_modifier = r_alt_modifier | l_ctrl_modifier;
 
-    observer_loop.trigger_server_actions();
-
     auto alt_down = key_event_builder_1->key_event(arbitrary_timestamp, mir_keyboard_action_down, 0, KEY_RIGHTALT);
     auto ctrl_down = key_event_builder_2->key_event(arbitrary_timestamp, mir_keyboard_action_down, 0, KEY_LEFTCTRL);
     auto ctrl_up = key_event_builder_2->key_event(arbitrary_timestamp, mir_keyboard_action_up, 0, KEY_LEFTCTRL);
@@ -416,8 +472,6 @@ TEST_F(SingleSeatInputDeviceHubSetup, tracks_a_single_cursor_position_from_multi
     hub.add_device(mt::fake_shared(device));
     hub.add_device(mt::fake_shared(another_device));
 
-    observer_loop.trigger_server_actions();
-
     auto motion_1 =
         mouse_event_builder_1->pointer_event(arbitrary_timestamp, mir_pointer_action_motion, 0, 0, 0, 23, 20);
     auto motion_2 =
@@ -449,8 +503,6 @@ TEST_F(SingleSeatInputDeviceHubSetup, tracks_a_single_button_state_from_multiple
     hub.add_device(mt::fake_shared(device));
     hub.add_device(mt::fake_shared(another_device));
 
-    observer_loop.trigger_server_actions();
-
     auto motion_1 =
         mouse_event_builder_1->pointer_event(arbitrary_timestamp, mir_pointer_action_button_down, mir_pointer_button_primary, 0, 0, 0, 0);
     auto motion_2 =
@@ -472,33 +524,36 @@ TEST_F(SingleSeatInputDeviceHubSetup, tracks_a_single_button_state_from_multiple
     mouse_sink_1->handle_input(*motion_4);
 }
 
-TEST_F(SingleSeatInputDeviceHubSetup, input_device_changes_sent_to_sink)
+TEST_F(SingleSeatInputDeviceHubSetup, input_device_changes_sent_to_session)
 {
-    EXPECT_CALL(mock_sink, handle_input_config_change(UnorderedElementsAre(DeviceMatches(device.get_device_info()))));
+    NiceMock<mtd::MockSceneSession> session;
+    stub_session_container.insert_session(mt::fake_shared(session));
+
+    EXPECT_CALL(session, send_input_config(UnorderedElementsAre(DeviceMatches(device.get_device_info()))));
     hub.add_device(mt::fake_shared(device));
-    observer_loop.trigger_server_actions();
 }
 
-TEST_F(SingleSeatInputDeviceHubSetup, input_device_changes_sent_to_sink_multiple_devices)
+TEST_F(SingleSeatInputDeviceHubSetup, input_device_changes_sent_to_session_multiple_devices)
 {
-    hub.add_device(mt::fake_shared(device));
-    observer_loop.trigger_server_actions();
+    NiceMock<mtd::MockSceneSession> session;
+    stub_session_container.insert_session(mt::fake_shared(session));
 
-    EXPECT_CALL(mock_sink,
-                handle_input_config_change(UnorderedElementsAre(DeviceMatches(device.get_device_info()),
-                                                                DeviceMatches(another_device.get_device_info()))));
+    hub.add_device(mt::fake_shared(device));
+
+    EXPECT_CALL(session,
+                send_input_config(UnorderedElementsAre(DeviceMatches(device.get_device_info()),
+                                                       DeviceMatches(another_device.get_device_info()))));
     hub.add_device(mt::fake_shared(another_device));
-    observer_loop.trigger_server_actions();
 }
 
 TEST_F(SingleSeatInputDeviceHubSetup, input_device_changes_sent_to_sink_removal)
 {
     hub.add_device(mt::fake_shared(device));
     hub.add_device(mt::fake_shared(another_device));
-    observer_loop.trigger_server_actions();
 
-    EXPECT_CALL(mock_sink,
-                handle_input_config_change(UnorderedElementsAre(DeviceMatches(another_device.get_device_info()))));
+    NiceMock<mtd::MockSceneSession> session;
+    stub_session_container.insert_session(mt::fake_shared(session));
+    EXPECT_CALL(session,
+                send_input_config(UnorderedElementsAre(DeviceMatches(another_device.get_device_info()))));
     hub.remove_device(mt::fake_shared(device));
-    observer_loop.trigger_server_actions();
 }
