@@ -2,7 +2,7 @@
  * Copyright © 2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -24,7 +24,6 @@
 #include "src/server/scene/basic_surface.h"
 #include "src/server/compositor/default_display_buffer_compositor_factory.h"
 #include "src/server/compositor/multi_threaded_compositor.h"
-#include "src/server/compositor/buffer_map.h"
 #include "src/server/compositor/stream.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/doubles/mock_buffer_stream.h"
@@ -32,9 +31,7 @@
 #include "mir/test/doubles/stub_renderer.h"
 #include "mir/test/doubles/stub_display_buffer.h"
 #include "mir/test/doubles/stub_buffer.h"
-#include "mir/test/doubles/stub_input_sender.h"
 #include "mir/test/doubles/null_display_sync_group.h"
-#include "mir/test/doubles/stub_frame_dropping_policy_factory.h"
 #include "mir/test/doubles/mock_event_sink.h"
 #include "mir/test/doubles/stub_buffer_allocator.h"
 
@@ -49,6 +46,7 @@ namespace mtd = mir::test::doubles;
 namespace mr = mir::report;
 namespace mc = mir::compositor;
 namespace mg = mir::graphics;
+namespace mf = mir::frontend;
 namespace geom = mir::geometry;
 using namespace testing;
 
@@ -125,8 +123,7 @@ struct SurfaceStackCompositor : public Test
 {
     SurfaceStackCompositor() :
         timeout{std::chrono::system_clock::now() + std::chrono::seconds(5)},
-        buffers(std::make_shared<mc::BufferMap>(std::make_shared<NiceMock<mtd::MockEventSink>>())),
-        stream(std::make_shared<mc::Stream>(policy_factory, buffers, geom::Size{ 1, 1 }, mir_pixel_format_abgr_8888 )),
+        stream(std::make_shared<mc::Stream>(geom::Size{ 1, 1 }, mir_pixel_format_abgr_8888 )),
         mock_buffer_stream(std::make_shared<NiceMock<mtd::MockBufferStream>>()),
         streams({ { stream, {0,0}, {} } }),
         stub_surface{std::make_shared<ms::BasicSurface>(
@@ -134,23 +131,18 @@ struct SurfaceStackCompositor : public Test
             geom::Rectangle{{0,0},{1,1}},
             mir_pointer_unconfined,
             streams,
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mtd::StubInputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             null_scene_report)},
         stub_buffer(std::make_shared<mtd::StubBuffer>())
     {
-        buffers->add_buffer(stub_buffer);
         ON_CALL(*mock_buffer_stream, lock_compositor_buffer(_))
             .WillByDefault(Return(mt::fake_shared(*stub_buffer)));
     }
-    mtd::StubFrameDroppingPolicyFactory policy_factory;
     std::shared_ptr<ms::SceneReport> null_scene_report{mr::null_scene_report()};
     ms::SurfaceStack stack{null_scene_report};
     std::shared_ptr<mc::CompositorReport> null_comp_report{mr::null_compositor_report()};
     StubRendererFactory renderer_factory;
     std::chrono::system_clock::time_point timeout;
-    std::shared_ptr<mc::BufferMap> buffers;
     std::shared_ptr<mc::Stream> stream;
     std::shared_ptr<mtd::MockBufferStream> mock_buffer_stream;
     std::list<ms::StreamInfo> const streams;
