@@ -1,8 +1,8 @@
 /*
- * Copyright © 2014 Canonical Ltd.
+ * Copyright © 2014-2017 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,12 +18,15 @@
 
 #include "mir_test_framework/async_server_runner.h"
 #include "mir_test_framework/command_line_server_configuration.h"
+#include "mir_test_framework/canonical_window_manager_policy.h"
 
 #include "mir/fd.h"
 #include "mir/main_loop.h"
 #include "mir/geometry/rectangle.h"
 #include "mir/options/option.h"
 #include "mir/test/doubles/null_logger.h"
+#include <miral/set_window_management_policy.h>
+#include <mir/shell/canonical_window_manager.h>
 
 #include <boost/throw_exception.hpp>
 
@@ -31,6 +34,7 @@
 
 namespace geom = mir::geometry;
 namespace ml = mir::logging;
+namespace msh = mir::shell;
 namespace mtd = mir::test::doubles;
 namespace mtf = mir_test_framework;
 namespace mt = mir::test;
@@ -40,7 +44,8 @@ namespace
 std::chrono::seconds const timeout{20};
 }
 
-mtf::AsyncServerRunner::AsyncServerRunner()
+mtf::AsyncServerRunner::AsyncServerRunner() :
+    set_window_management_policy{[](auto&){}}
 {
     configure_from_commandline(server);
 
@@ -54,6 +59,18 @@ mtf::AsyncServerRunner::AsyncServerRunner()
 
             return result;
         });
+    // TODO This is here to support tests that rely on the legacy window management code
+    // once they go, then we can set set_window_management_policy appropriately, kill this
+    // and remove msh::CanonicalWindowManager
+    server.override_the_window_manager_builder([this](msh::FocusController* focus_controller)
+        {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+            return std::make_shared<msh::CanonicalWindowManager>(
+                    focus_controller,
+                    server.the_shell_display_layout());
+#pragma GCC diagnostic pop
+        });
 }
 
 void mtf::AsyncServerRunner::add_to_environment(char const* key, char const* value)
@@ -63,6 +80,8 @@ void mtf::AsyncServerRunner::add_to_environment(char const* key, char const* val
 
 void mtf::AsyncServerRunner::start_server()
 {
+    set_window_management_policy(server);
+
     server.add_init_callback([&]
         {
             auto const main_loop = server.the_main_loop();
