@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 3,
+ * under the terms of the GNU General Public License version 2 or 3,
  * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -55,7 +55,7 @@ public:
         mir::SharedLibrary stub_platform_library{std::string(platform_path) + "/graphics-dummy.so"};
         auto create_stub_platform = stub_platform_library.load_function<mg::CreateHostPlatform>("create_host_platform");
 
-        stub_platform = create_stub_platform(nullptr, nullptr, nullptr);
+        stub_platform = create_stub_platform(nullptr, nullptr, nullptr, nullptr);
     }
 
     mir::UniqueModulePtr<mg::GraphicBufferAllocator> create_buffer_allocator() override
@@ -84,13 +84,36 @@ public:
         return stub_platform->make_ipc_operations();
     }
 
+    mg::NativeRenderingPlatform* native_rendering_platform() override
+    {
+        if (should_throw.at(ExceptionLocation::at_native_rendering_platform))
+            BOOST_THROW_EXCEPTION(std::runtime_error("Exception during egl_native_display"));
+
+        return stub_platform->native_rendering_platform();
+    }
+
+    mg::NativeDisplayPlatform* native_display_platform() override
+    {
+        if (should_throw.at(ExceptionLocation::at_native_display_platform))
+            BOOST_THROW_EXCEPTION(std::runtime_error("Exception during egl_native_display"));
+
+        return stub_platform->native_display_platform();
+    }
+
+    std::vector<mir::ExtensionDescription> extensions() const
+    {
+        return {};
+    }
+
 private:
     enum ExceptionLocation : uint32_t
     {
         at_constructor,
         at_create_buffer_allocator,
         at_create_display,
-        at_make_ipc_operations
+        at_make_ipc_operations,
+        at_native_rendering_platform,
+        at_native_display_platform,
     };
 
     static std::unordered_map<ExceptionLocation, bool, std::hash<uint32_t>> parse_exception_request(char const* request)
@@ -104,6 +127,10 @@ private:
             static_cast<bool>(strstr(request, "create_display"));
         requested_exceptions[ExceptionLocation::at_make_ipc_operations] =
             static_cast<bool>(strstr(request, "make_ipc_operations"));
+        requested_exceptions[ExceptionLocation::at_native_rendering_platform] =
+            static_cast<bool>(strstr(request, "native_rendering_platform"));
+        requested_exceptions[ExceptionLocation::at_native_display_platform] =
+            static_cast<bool>(strstr(request, "native_display_platform"));
 
         return requested_exceptions;
     };
@@ -142,16 +169,10 @@ void add_graphics_platform_options(boost::program_options::options_description&)
 mir::UniqueModulePtr<mg::Platform> create_host_platform(
     std::shared_ptr<mo::Option> const&,
     std::shared_ptr<mir::EmergencyCleanupRegistry> const&,
-    std::shared_ptr<mg::DisplayReport> const&)
+    std::shared_ptr<mg::DisplayReport> const&,
+    std::shared_ptr<mir::logging::Logger> const&)
 {
     mir::assert_entry_point_signature<mg::CreateHostPlatform>(&create_host_platform);
     return mir::make_module_ptr<ExceptionThrowingPlatform>();
 }
 
-mir::UniqueModulePtr<mg::Platform> create_guest_platform(
-    std::shared_ptr<mg::DisplayReport> const&,
-    std::shared_ptr<mg::NestedContext> const&)
-{
-    mir::assert_entry_point_signature<mg::CreateGuestPlatform>(&create_guest_platform);
-    return mir::make_module_ptr<ExceptionThrowingPlatform>();
-}
