@@ -2,7 +2,7 @@
  * Copyright © 2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  */
 
 #include "src/client/screencast_stream.h"
-#include "mir/client_platform.h"
+#include "mir/client/client_platform.h"
 #include "mir/test/doubles/null_client_buffer.h"
 #include "mir/test/doubles/mock_client_buffer_factory.h"
 #include "mir/test/doubles/mock_protobuf_server.h"
@@ -78,9 +78,11 @@ struct ScreencastStream : Test
         ON_CALL(mock_factory, create_buffer(_,An<geom::Size>(),_))
             .WillByDefault(Return(std::make_shared<mtd::NullClientBuffer>()));
     }
-
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     mp::BufferStream a_protobuf_buffer_stream(MirPixelFormat format, MirBufferUsage usage, MirBufferPackage const& package)
     {
+#pragma GCC diagnostic pop
         mp::BufferStream protobuf_bs;
         mp::BufferStreamId bs_id;
         
@@ -100,8 +102,10 @@ struct ScreencastStream : Test
     testing::NiceMock<mtd::MockProtobufServer> mock_protobuf_server;
 
     MirPixelFormat const default_pixel_format = mir_pixel_format_argb_8888;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     MirBufferUsage const default_buffer_usage = mir_buffer_usage_hardware;
-
+#pragma GCC diagnostic pop
     MirBufferPackage buffer_package = a_buffer_package();
     geom::Size size{buffer_package.width, buffer_package.height};
     mp::BufferStream response = a_protobuf_buffer_stream(
