@@ -2,7 +2,7 @@
  * Copyright © 2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -24,9 +24,10 @@
 #include "mir/time/alarm_factory.h"
 #include "mir/cookie/authority.h"
 #include "mir/input/input_device_observer.h"
-#include "mir/input/pointer_configuration.h"
-#include "mir/input/touchpad_configuration.h"
-#include "mir/input/keyboard_configuration.h"
+#include "mir/input/mir_pointer_config.h"
+#include "mir/input/mir_touchpad_config.h"
+#include "mir/input/mir_touchscreen_config.h"
+#include "mir/input/mir_keyboard_config.h"
 #include "mir/input/device.h"
 
 #include "mir/test/fake_shared.h"
@@ -91,12 +92,14 @@ struct StubDevice : public mi::Device
     std::string name() const {return device_name;}
     std::string unique_id() const {return {};}
 
-    mir::optional_value<mi::PointerConfiguration> pointer_configuration() const {return {};}
-    void apply_pointer_configuration(mi::PointerConfiguration const&) {;}
-    mir::optional_value<mi::TouchpadConfiguration> touchpad_configuration() const {return {};}
-    void apply_touchpad_configuration(mi::TouchpadConfiguration const&) {}
-    mir::optional_value<mi::KeyboardConfiguration> keyboard_configuration() const {return {};}
-    void apply_keyboard_configuration(mi::KeyboardConfiguration const&) {}
+    mir::optional_value<MirPointerConfig> pointer_configuration() const {return {};}
+    void apply_pointer_configuration(MirPointerConfig const&) {;}
+    mir::optional_value<MirTouchpadConfig> touchpad_configuration() const {return {};}
+    void apply_touchpad_configuration(MirTouchpadConfig const&) {}
+    mir::optional_value<MirKeyboardConfig> keyboard_configuration() const {return {};}
+    void apply_keyboard_configuration(MirKeyboardConfig const&) {}
+    mir::optional_value<MirTouchscreenConfig> touchscreen_configuration() const {return {};}
+    void apply_touchscreen_configuration(MirTouchscreenConfig const&) {}
 };
 
 struct KeyRepeatDispatcher : public testing::Test
