@@ -2,7 +2,7 @@
  * Copyright © 2013-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -29,8 +29,6 @@
 
 #include "mir/test/doubles/stub_cursor_image.h"
 #include "mir/test/doubles/mock_buffer_stream.h"
-#include "mir/test/doubles/mock_input_sender.h"
-#include "mir/test/doubles/stub_input_sender.h"
 #include "mir/test/doubles/stub_buffer.h"
 #include "mir/test/fake_shared.h"
 
@@ -64,7 +62,7 @@ public:
 class MockSurfaceObserver : public ms::NullSurfaceObserver
 {
 public:
-    MOCK_METHOD2(attrib_changed, void(MirSurfaceAttrib, int));
+    MOCK_METHOD2(attrib_changed, void(MirWindowAttrib, int));
     MOCK_METHOD1(hidden_set_to, void(bool));
     MOCK_METHOD1(renamed, void(char const*));
     MOCK_METHOD0(client_surface_close_requested, void());
@@ -86,20 +84,22 @@ struct BasicSurfaceTest : public testing::Test
     void const* compositor_id{nullptr};
     std::shared_ptr<ms::LegacySurfaceChangeNotification> observer =
         std::make_shared<ms::LegacySurfaceChangeNotification>(mock_change_cb, [this](int){mock_change_cb();});
-    std::shared_ptr<mi::InputSender> const stub_input_sender = std::make_shared<mtd::StubInputSender>();
-    testing::NiceMock<mtd::MockInputSender> mock_sender;
     std::list<ms::StreamInfo> streams { { mock_buffer_stream, {}, {} } };
 
     ms::BasicSurface surface{
         name,
         rect,
         mir_pointer_unconfined,
-        false,
         streams,
-        std::shared_ptr<mi::InputChannel>(),
-        stub_input_sender,
         std::shared_ptr<mg::CursorImage>(),
         report};
+
+    BasicSurfaceTest()
+    {
+        // use an opaque pixel format by default
+        ON_CALL(*mock_buffer_stream, pixel_format())
+            .WillByDefault(testing::Return(mir_pixel_format_xrgb_8888));
+    }
 };
 
 }
@@ -122,10 +122,9 @@ TEST_F(BasicSurfaceTest, buffer_stream_ids_always_unique)
     for (auto& surface : surfaces)
     {
         surface = std::make_unique<ms::BasicSurface>(
-                name, rect, mir_pointer_unconfined, false, 
+                name, rect, mir_pointer_unconfined,
                 std::list<ms::StreamInfo> {
                     { std::make_shared<testing::NiceMock<mtd::MockBufferStream>>(), {}, {} } },
-                std::shared_ptr<mi::InputChannel>(), stub_input_sender,
                 std::shared_ptr<mg::CursorImage>(), report);
         for (auto& renderable : surface->generate_renderables(this))
             ids.insert(renderable->id());
@@ -143,8 +142,7 @@ TEST_F(BasicSurfaceTest, id_never_invalid)
     for (auto& surface : surfaces)
     {
         surface = std::make_unique<ms::BasicSurface>(
-                name, rect, mir_pointer_unconfined, false, streams,
-                std::shared_ptr<mi::InputChannel>(), stub_input_sender,
+                name, rect, mir_pointer_unconfined, streams,
                 std::shared_ptr<mg::CursorImage>(), report);
 
         for (auto& renderable : surface->generate_renderables(this))
@@ -231,24 +229,10 @@ TEST_F(BasicSurfaceTest, test_surface_set_transformation_updates_transform)
     EXPECT_EQ(trans, got);
 }
 
-TEST_F(BasicSurfaceTest, test_surface_set_alpha_notifies_changes)
-{
-    using namespace testing;
-    EXPECT_CALL(mock_callback, call())
-        .Times(1);
-
-    surface.add_observer(observer);
-
-    float alpha = 0.5f;
-    surface.set_alpha(0.5f);
-    EXPECT_THAT(alpha, FloatEq(surface.alpha()));
-}
-
 TEST_F(BasicSurfaceTest, test_surface_is_opaque_by_default)
 {
     using namespace testing;
 
-    EXPECT_THAT(1.0f, FloatEq(surface.alpha()));
     auto renderables = surface.generate_renderables(compositor_id);
     ASSERT_THAT(renderables.size(), testing::Eq(1));
     EXPECT_FALSE(renderables[0]->shaped());
@@ -267,10 +251,7 @@ TEST_F(BasicSurfaceTest, test_surface_visibility)
         name,
         rect,
         mir_pointer_unconfined,
-        false,
         streams,
-        std::shared_ptr<mi::InputChannel>(),
-        stub_input_sender,
         std::shared_ptr<mg::CursorImage>(),
         report};
 
@@ -308,10 +289,7 @@ TEST_F(BasicSurfaceTest, default_region_is_surface_rectangle)
         name,
         geom::Rectangle{pt, one_by_one},
         mir_pointer_unconfined,
-        false,
         streams,
-        std::shared_ptr<mi::InputChannel>(),
-        stub_input_sender,
         std::shared_ptr<mg::CursorImage>(),
         report};
 
@@ -350,10 +328,7 @@ TEST_F(BasicSurfaceTest, default_invisible_surface_doesnt_get_input)
         name,
         geom::Rectangle{{0,0}, {100,100}},
         mir_pointer_unconfined,
-        false,
         streams,
-        std::shared_ptr<mi::InputChannel>(),
-        stub_input_sender,
         std::shared_ptr<mg::CursorImage>(),
         report};
 
@@ -491,10 +466,7 @@ TEST_F(BasicSurfaceTest, stores_parent)
         geom::Rectangle{{0,0}, {100,100}},
         parent,
         mir_pointer_unconfined,
-        false,
         streams,
-        std::shared_ptr<mi::InputChannel>(),
-        stub_input_sender,
         std::shared_ptr<mg::CursorImage>(),
         report};
 
@@ -506,7 +478,7 @@ namespace
 
 struct AttributeTestParameters
 {
-    MirSurfaceAttrib attribute;
+    MirWindowAttrib attribute;
     int default_value;
     int a_valid_value;
     int an_invalid_value;
@@ -518,44 +490,44 @@ struct BasicSurfaceAttributeTest : public BasicSurfaceTest,
 };
 
 AttributeTestParameters const surface_visibility_test_parameters{
-    mir_surface_attrib_visibility,
-    mir_surface_visibility_occluded,
-    mir_surface_visibility_exposed,
+    mir_window_attrib_visibility,
+    mir_window_visibility_occluded,
+    mir_window_visibility_exposed,
     -1
 };
 
 AttributeTestParameters const surface_type_test_parameters{
-    mir_surface_attrib_type,
-    mir_surface_type_normal,
-    mir_surface_type_freestyle,
+    mir_window_attrib_type,
+    mir_window_type_normal,
+    mir_window_type_freestyle,
     -1
 };
 
 AttributeTestParameters const surface_state_test_parameters{
-    mir_surface_attrib_state,
-    mir_surface_state_restored,
-    mir_surface_state_fullscreen,
+    mir_window_attrib_state,
+    mir_window_state_restored,
+    mir_window_state_fullscreen,
     1178312
 };
 
 AttributeTestParameters const surface_swapinterval_test_parameters{
-    mir_surface_attrib_swapinterval,
+    mir_window_attrib_swapinterval,
     1,
     0,
     -1
 };
 
 AttributeTestParameters const surface_dpi_test_parameters{
-    mir_surface_attrib_dpi,
+    mir_window_attrib_dpi,
     0,
     90,
     -1
 };
 
 AttributeTestParameters const surface_focus_test_parameters{
-    mir_surface_attrib_focus,
-    mir_surface_unfocused,
-    mir_surface_focused,
+    mir_window_attrib_focus,
+    mir_window_focus_state_unfocused,
+    mir_window_focus_state_focused,
     -1
 };
 
@@ -671,26 +643,6 @@ TEST_F(BasicSurfaceTest, notifies_about_cursor_image_removal)
     surface.set_cursor_image({});
 }
 
-TEST_F(BasicSurfaceTest, calls_send_event_on_consume)
-{
-    using namespace ::testing;
-
-    ms::BasicSurface surface{
-        name,
-        rect,
-        mir_pointer_unconfined,
-        false,
-        streams,
-        std::shared_ptr<mi::InputChannel>(),
-        mt::fake_shared(mock_sender),
-        nullptr,
-        report};
-
-    EXPECT_CALL(mock_sender, send_event(_,_));
-
-    surface.consume(mev::make_event(mir_prompt_session_state_started).get());
-}
-
 TEST_F(BasicSurfaceTest, observer_can_trigger_state_change_within_notification)
 {
     using namespace testing;
@@ -919,9 +871,9 @@ TEST_F(BasicSurfaceTest, showing_brings_all_streams_up_to_date)
     EXPECT_CALL(*buffer_stream, drop_old_buffers()).Times(Exactly(1));
     EXPECT_CALL(*mock_buffer_stream, drop_old_buffers()).Times(Exactly(1));
 
-    surface.configure(mir_surface_attrib_visibility, mir_surface_visibility_occluded);
-    surface.configure(mir_surface_attrib_visibility, mir_surface_visibility_exposed);
-    surface.configure(mir_surface_attrib_visibility, mir_surface_visibility_exposed);
+    surface.configure(mir_window_attrib_visibility, mir_window_visibility_occluded);
+    surface.configure(mir_window_attrib_visibility, mir_window_visibility_exposed);
+    surface.configure(mir_window_attrib_visibility, mir_window_visibility_exposed);
 }
 
 //TODO: per-stream alpha and swapinterval seems useful
@@ -987,7 +939,7 @@ TEST_F(BasicSurfaceTest, changing_inverval_effects_all_streams)
     EXPECT_CALL(*buffer_stream, allow_framedropping(true));
 
     surface.set_streams(streams);
-    surface.configure(mir_surface_attrib_swapinterval, 0);
+    surface.configure(mir_window_attrib_swapinterval, 0);
 }
 
 TEST_F(BasicSurfaceTest, visibility_matches_produced_list)
@@ -1076,17 +1028,38 @@ TEST_F(BasicSurfaceTest, buffer_streams_produce_correctly_sized_renderables)
     EXPECT_THAT(renderables[1], IsRenderableOfSize(size1));
 }
 
+TEST_F(BasicSurfaceTest, renderables_of_transparent_buffer_streams_are_shaped)
+{
+    using namespace testing;
+    auto buffer_stream = std::make_shared<NiceMock<mtd::MockBufferStream>>();
+    geom::Displacement d0{0,0};
+    geom::Displacement d1{19,99};
+    ON_CALL(*buffer_stream, pixel_format())
+        .WillByDefault(Return(mir_pixel_format_argb_8888));
+
+    std::list<ms::StreamInfo> streams = {
+        { mock_buffer_stream, d0, {} },
+        { buffer_stream, d1, {} },
+    };
+    surface.set_streams(streams);
+
+    auto renderables = surface.generate_renderables(this);
+    ASSERT_THAT(renderables.size(), Eq(2));
+    EXPECT_THAT(renderables[0]->shaped(), false);
+    EXPECT_THAT(renderables[1]->shaped(), true);
+}
+
 namespace
 {
 struct VisibilityObserver : ms::NullSurfaceObserver
 {
-    void attrib_changed(MirSurfaceAttrib attrib, int value) override
+    void attrib_changed(MirWindowAttrib attrib, int value) override
     {
-        if (attrib == mir_surface_attrib_visibility)
+        if (attrib == mir_window_attrib_visibility)
         {
-            if (value == mir_surface_visibility_occluded)
+            if (value == mir_window_visibility_occluded)
                 hides_++;
-            else if (value == mir_surface_visibility_exposed)
+            else if (value == mir_window_visibility_exposed)
                 exposes_++;
         }
     }
@@ -1112,7 +1085,7 @@ TEST_F(BasicSurfaceTest, notifies_when_first_visible)
 
     EXPECT_THAT(observer->exposes(), Eq(0));
     EXPECT_THAT(observer->hides(), Eq(0));
-    surface.configure(mir_surface_attrib_visibility, mir_surface_visibility_exposed);
+    surface.configure(mir_window_attrib_visibility, mir_window_visibility_exposed);
 
     EXPECT_THAT(observer->exposes(), Eq(1));
     EXPECT_THAT(observer->hides(), Eq(0));
