@@ -2,7 +2,7 @@
  * Copyright © 2012-2014 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -22,18 +22,14 @@
 #include "mir/scene/observer.h"
 #include "mir/scene/surface_creation_parameters.h"
 #include "mir/compositor/scene_element.h"
-#include "mir/compositor/decoration.h"
 #include "src/server/report/null_report_factory.h"
 #include "src/server/scene/basic_surface.h"
 #include "src/server/compositor/stream.h"
-#include "mir/input/input_channel_factory.h"
-#include "mir/test/doubles/stub_input_channel.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/doubles/stub_buffer_stream.h"
 #include "mir/test/doubles/stub_buffer_stream_factory.h"
 #include "mir/test/doubles/stub_renderable.h"
 #include "mir/test/doubles/mock_buffer_stream.h"
-#include "mir/test/doubles/stub_frame_dropping_policy_factory.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -72,35 +68,6 @@ MATCHER_P(SceneElementForStream, stream, "")
     return arg->renderable()->id() == stream.get();
 }
 
-struct StubInputChannelFactory : public mi::InputChannelFactory
-{
-    std::shared_ptr<mi::InputChannel> make_input_channel()
-    {
-        return std::make_shared<mtd::StubInputChannel>();
-    }
-};
-
-struct StubInputChannel : public mi::InputChannel
-{
-    StubInputChannel(int server_fd, int client_fd)
-        : s_fd(server_fd),
-          c_fd(client_fd)
-    {
-    }
-
-    int client_fd() const
-    {
-        return c_fd;
-    }
-    int server_fd() const
-    {
-        return s_fd;
-    }
-
-    int const s_fd;
-    int const c_fd;
-};
-
 struct MockCallback
 {
     MOCK_METHOD0(call, void());
@@ -129,8 +96,6 @@ struct SurfaceStack : public ::testing::Test
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { stub_buffer_stream1, {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
 
@@ -139,8 +104,6 @@ struct SurfaceStack : public ::testing::Test
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { stub_buffer_stream2, {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
 
@@ -150,8 +113,6 @@ struct SurfaceStack : public ::testing::Test
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { stub_buffer_stream3, {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
 
@@ -161,8 +122,6 @@ struct SurfaceStack : public ::testing::Test
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
         invisible_stub_surface->set_hidden(true);
@@ -266,93 +225,19 @@ TEST_F(SurfaceStack, scene_snapshot_omits_invisible_surfaces)
             SceneElementForStream(stub_buffer_stream2)));
 }
 
-TEST_F(SurfaceStack, decor_name_is_surface_name)
-{
-    using namespace testing;
-
-    ms::SurfaceStack stack{report};
-    auto surface = std::make_shared<ms::BasicSurface>(
-        std::string("Mary had a little lamb"),
-        geom::Rectangle{{},{}},
-        mir_pointer_unconfined,
-        std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
-        std::shared_ptr<mg::CursorImage>(),
-        report);
-    stack.add_surface(surface, default_params.input_mode);
-    surface->configure(mir_window_attrib_visibility,
-                       mir_window_visibility_exposed);
-    
-    auto elements = stack.scene_elements_for(compositor_id);
-    ASSERT_EQ(1, elements.size());
-
-    auto& element = elements.front();
-
-    auto decor = element->decoration();
-    ASSERT_THAT(decor, Ne(nullptr));
-    EXPECT_EQ(mc::Decoration::Type::surface, decor->type);
-    EXPECT_EQ("Mary had a little lamb", decor->name);
-}
-
-TEST_F(SurfaceStack, gets_surface_renames)
-{
-    using namespace testing;
-
-    ms::SurfaceStack stack{report};
-    auto surface = std::make_shared<ms::BasicSurface>(
-        std::string("username@hostname: /"),
-        geom::Rectangle{{},{}},
-        mir_pointer_unconfined,
-        std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
-        std::shared_ptr<mg::CursorImage>(),
-        report);
-    stack.add_surface(surface, default_params.input_mode);
-    surface->configure(mir_window_attrib_visibility,
-                       mir_window_visibility_exposed);
-    
-    // (change directory in shell app)
-    surface->rename("username@hostname: ~/Documents");
-
-    auto elements = stack.scene_elements_for(compositor_id);
-    ASSERT_EQ(1, elements.size());
-
-    auto& element = elements.front();
-
-    auto decor = element->decoration();
-    ASSERT_THAT(decor, Ne(nullptr));
-    EXPECT_EQ(mc::Decoration::Type::surface, decor->type);
-    EXPECT_EQ("username@hostname: ~/Documents", decor->name);
-}
-
 TEST_F(SurfaceStack, scene_counts_pending_accurately)
 {
     using namespace testing;
     ms::SurfaceStack stack{report};
     stack.register_compositor(this);
 
-    struct StubBuffers : mtd::StubClientBuffers
-    {
-        std::shared_ptr<mg::Buffer>& operator[](mg::BufferID) override
-        {
-            return buffer;
-        }
-        std::shared_ptr<mg::Buffer> buffer {std::make_shared<mtd::StubBuffer>()};
-    };
-
-    auto buffers = std::make_shared<StubBuffers>();
-    mtd::StubFrameDroppingPolicyFactory factory;
-    auto stream = std::make_shared<mc::Stream>(factory, buffers, geom::Size{ 1, 1 }, mir_pixel_format_abgr_8888);
+    auto stream = std::make_shared<mc::Stream>(geom::Size{ 1, 1 }, mir_pixel_format_abgr_8888);
 
     auto surface = std::make_shared<ms::BasicSurface>(
         std::string("stub"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
     stack.add_surface(surface, default_params.input_mode);
@@ -386,8 +271,6 @@ TEST_F(SurfaceStack, scene_doesnt_count_pending_frames_from_occluded_surfaces)
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
 
@@ -420,8 +303,6 @@ TEST_F(SurfaceStack, scene_doesnt_count_pending_frames_from_partially_exposed_su
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
 
@@ -534,8 +415,6 @@ TEST_F(SurfaceStack, generate_elementelements)
             geom::Rectangle{geom::Point{3 * i, 4 * i},geom::Size{1 * i, 2 * i}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
         
@@ -719,8 +598,6 @@ TEST_F(SurfaceStack, scene_elements_hold_snapshot_of_positioning_info)
             geom::Rectangle{geom::Point{3 * i, 4 * i},geom::Size{1 * i, 2 * i}},
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
-            std::shared_ptr<mir::input::InputChannel>(),
-            std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
 
@@ -752,8 +629,6 @@ TEST_F(SurfaceStack, generates_scene_elements_that_delay_buffer_acquisition)
         geom::Rectangle{geom::Point{3, 4},geom::Size{1, 2}},
         mir_pointer_unconfined,
         std::list<ms::StreamInfo> { { mock_stream, {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
         stack.add_surface(surface, default_params.input_mode);
@@ -782,8 +657,6 @@ TEST_F(SurfaceStack, generates_scene_elements_that_allow_only_one_buffer_acquisi
         geom::Rectangle{geom::Point{3, 4},geom::Size{1, 2}},
         mir_pointer_unconfined,
         std::list<ms::StreamInfo> { { mock_stream, {}, {} } },
-        std::shared_ptr<mir::input::InputChannel>(),
-        std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
         stack.add_surface(surface, default_params.input_mode);
@@ -806,8 +679,6 @@ struct MockConfigureSurface : public ms::BasicSurface
             mir_pointer_unconfined,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
             {},
-            {},
-            {},
             mir::report::null_scene_report())
     {
     }
@@ -1048,7 +919,7 @@ TEST_F(SurfaceStack, scene_observers_notified_of_generic_scene_change)
     stack.emit_scene_changed();
 }
 
-TEST_F(SurfaceStack, only_enumerates_exposed_input_surfaces)
+TEST_F(SurfaceStack, for_each_enumerates_all_input_surfaces)
 {
     using namespace ::testing;
 
@@ -1066,7 +937,7 @@ TEST_F(SurfaceStack, only_enumerates_exposed_input_surfaces)
     };
 
     stack.for_each(count_exposed_surfaces);
-    EXPECT_THAT(num_exposed_surfaces, Eq(1));
+    EXPECT_THAT(num_exposed_surfaces, Eq(3));
 }
 
 using namespace ::testing;
