@@ -2,7 +2,7 @@
  * Copyright © 2014-2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
+ * it under the terms of the GNU General Public License version 2 or 3 as
  * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -185,7 +185,7 @@ TEST_F(GLibMainLoopTest, handles_multiple_signals)
 
 
     std::thread signal_sending_thread(
-        [this, num_signals_to_send, &signals, &num_handled_signals]
+        [this, &signals, &num_handled_signals]
         {
             for (size_t i = 0; i < num_signals_to_send; i++)
             {
@@ -352,7 +352,7 @@ TEST_F(GLibMainLoopTest, multiple_fds_with_single_handler_handled)
         });
 
     std::thread fd_writing_thread{
-        [this, num_elems_to_send, &pipes, &num_handled_fds]
+        [this, &pipes, &num_handled_fds]
         {
             for (size_t i = 0; i < num_elems_to_send; i++)
             {
